/*
   NAME:        Panic.cc
   AUTHOR:      Ole Gunnar Westgaard (olew@ifi.uio.no)
   DATE:        Feb  1 1997
   COMMENT:     Panic - Sound Editor
*/

#include <stdio.h>
#include "otools.h"
#include "figlets.h"
#include "icon.xbm"

FILEDIALOG *files = NULL;

void Quit(char *s)
{
  fprintf(stderr,"%s\n",s);
  exit(0);
}

class MAINWINDOW : public WINDOW {
protected:
  COLORSET *menucolors;
  MENU *menu;
public:
  MAINWINDOW(COLORSET *colors,char *font,ITEM *items)
    : WINDOW(0,NULL,colors,"Panic soundeditor v1.0","panic",
	     10,10,100,10,320,60,
	     colors->cmap()->dpy()->Width(),
	     colors->cmap()->dpy()->Height(),0,
	     icon_bits,icon_width,icon_height,font)
  {
    if (!display) return;
    menucolors = new COLORSET(colorset->cmap(),
      "black:gray50:gray70:gray90:gray20:red4:red1:cyan4:black");
    if (!menucolors->cmap()) Quit("Error: Cannot create menus colorset!");
    menu = new MENU(this,menucolors,0,items);
    Size();
  };
  virtual void Size(void)
  {
    //IBUG("Size()%c%c%c",0,0,0);
    menu->Resize();
    Resize(menu->pixel_width(),menu->pixel_height());
    //Refresh(0,0,menu->pixel_width(),menu->pixel_height());
    menu->Refresh();
    //OBUG("Size()%c%c%c",0,0,0);
  };
  void Font(char *f)
  {
    menu->kill_submenu();
    pen->Font(f);
    Size();
  };
  void Color(char *a,char *b,char *c,char *d,char *e,char *f,char *g,char *h)
  {
    menu->kill_submenu();
    menucolors->Change(ocSHADOW,a);
    menucolors->Change(ocBACKGROUND,b);
    menucolors->Change(ocHIGHLIGHT,c);
    menucolors->Change(ocFOREGROUND,d);
    menucolors->Change(ocDARK,e);
    menucolors->Change(ocLIGHT,f);
    menucolors->Change(ocDISABLED,g);
    menucolors->Change(ocSELECTED,h);
    menu->Refresh();
  };
};

COLORSET   *colorset;
MAINWINDOW *application;

void OpenFile(char*)
{
  files->show();
  application->dpy()->Flush();
}

void StopOtools(char*)
{
  otools.Stop();
}

void MenuFont(char *font)
{
  if (strcmp(font,"10 point") == 0)
    application->Font("6x10"); else
  if (strcmp(font,"13 point") == 0)
    application->Font("7x13bold"); else
  if (strcmp(font,"12 point") == 0)
    application->Font("6x12"); else
  if (strcmp(font,"15 point") == 0)
    application->Font("-*-*-bold-r-*-*-15-*-*-*-*-*-iso8859-1"); else
  if (strcmp(font,"19 point") == 0)
    application->Font("-*-*-bold-r-*-*-19-*-*-*-*-*-iso8859-1");
}

void MenuColor(char *color)
{
  if (strcmp(color,"Gray") == 0) application->Color(
	"gray50","gray70","gray90","gray20","red4","red1","cyan4","black");
  else
  if (strcmp(color,"Cyan") == 0) application->Color(
	"cyan4","cyan3","cyan1","black","blue","yellow","cyan4","white");
  else
  if (strcmp(color,"Green") == 0) application->Color(
	"green4","green3","green1","black","blue","yellow","green4","white");

}

int main(int,char**)
{ 
  DISPLAY display(NULL);
  if (!display.xdpy()) Quit("Error: Cannot connect to display!");

  colorset = new COLORSET(display.cmap(),
			  "black:gray50:gray70:gray90:white:black:blue");
  if (!colorset->cmap()) Quit("Error: Cannot create application colorset!");

  application = new MAINWINDOW(colorset,"-*-*-bold-r-*-*-14-*-*-*-*-*-*-*",

      SUB2("File",OPT2("Open",OpenFile,
		       "Quit",StopOtools
		       ),
	   "Menu",SUB2("Colors",OPT3("Gray",MenuColor,
				     "Cyan",MenuColor,
				     "Green",MenuColor),
		       "Font",OPT5("10 point",MenuFont,
				   "12 point",MenuFont,
				   "13 point",MenuFont,
				   "15 point",MenuFont,
				   "19 point",MenuFont)
		       )
	   )
  );

  files = new FILEDIALOG(colorset,60,60,320,200,"Open file","Load","*.snd",
			 10,3,0,"7x13bold");

  if (!application->dpy()) Quit("Error: Cannot create application window!");
  application->Size();

  otools.EventLoop();

  delete files;
  files = NULL;

  return 0;
}
