/*
   NAME:        Matrix.h
   AUTHOR:      root <root@>
   DATE:        Jan 24 1997
   COMMENT:     Matrix class
*/
#ifndef MATRIXH
#define MATRIXH

#include "Vector.h"

class Matrix {
private:
  float values[3][3]; // Rad, Kolonne (!!!!!)
public:
  inline Matrix(void) {
    values[0][0] = values[0][1] = values[0][2] = 0.0f;
    memcpy(values[1],values[0],3*sizeof(float));
    memcpy(values[2],values[0],3*sizeof(float));
  };
  inline Matrix(float *src) {
    memcpy(values[0],src  ,3*sizeof(float));
    memcpy(values[1],src+3,3*sizeof(float));
    memcpy(values[2],src+6,3*sizeof(float));
  };
  inline Matrix(Matrix &src) {
    memcpy(values[0],src.getrow(0),3*sizeof(float));
    memcpy(values[1],src.getrow(1),3*sizeof(float));
    memcpy(values[2],src.getrow(2),3*sizeof(float));
  };
  inline Matrix(Vector &A,Vector &B,Vector &C) {
    memcpy(values[0],A.array(),3*sizeof(float));
    memcpy(values[1],B.array(),3*sizeof(float));
    memcpy(values[2],C.array(),3*sizeof(float));
  };
  inline Matrix &operator=(Matrix src) {
    memcpy(values[0],src.getrow(0),3*sizeof(float));
    memcpy(values[1],src.getrow(1),3*sizeof(float));
    memcpy(values[2],src.getrow(2),3*sizeof(float));
    return *this;
  };
  inline Vector &operator[](short col) {
    return *(new Vector(values[col]));
  };
  inline Vector &operator()(short row) {
    return *(new Vector(values[0][row],values[1][row],values[2][row]));
  };
  inline float *getrow(short index) { // get row float array
    return values[index];
  };
  inline void normalize(float scale) {
    for (short row=0; row<3; row++)
      for (short col=0; col<3; col++)
	values[row][col] /= scale;    
  };
  inline void show(char *s) {
    fprintf(stderr,"+- Matrix ------------------------------+\n");
    fprintf(stderr,"| %-10s %8.3f,%8.3f,%8.3f |\n",s,
	    values[0][0],values[0][1],values[0][2]);
    fprintf(stderr,"|            %8.3f,%8.3f,%8.3f |\n",
	    values[1][0],values[1][1],values[1][2]);
    fprintf(stderr,"|            %8.3f,%8.3f,%8.3f |\n",
	    values[2][0],values[2][1],values[2][2]);
    fprintf(stderr,"+---------------------------------------+\n");
  };
};

inline Vector &operator*(Matrix M,Vector V) {
  return *(new Vector(M[0]*V,M[1]*V,M[2]*V));
}

inline Matrix &operator*(Matrix M1,Matrix M2) {
  float val[9];
  val[0] = M1[0] * M2(0);
  val[1] = M1[0] * M2(1);
  val[2] = M1[0] * M2(2);
  val[3] = M1[1] * M2(0);
  val[4] = M1[1] * M2(1);
  val[5] = M1[1] * M2(2);
  val[6] = M1[2] * M2(0);
  val[7] = M1[2] * M2(1);
  val[8] = M1[2] * M2(2);
  return *(new Matrix(val));
}

#endif /* MATRIXH */
