/*
   NAME:	arrow.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Dec 17 1996
   COMMENT:	ARROW, ThreeD arrow button
*/

#include "arrow.h"

ARROW::ARROW(WINDOW *w,int a,int b,int c,int d,uchar tp,uchar ar,uchar sp)
  :FIGLET(w,a,b,c,d)
{
  colors = NULL;
  pen = window->dpen();
  set = window->cset();
  type = tp;
  arrow = ar;
  space = sp;
  window->Add(this);
  window->Activate(this);
}

ARROW::ARROW(WINDOW *w,int a,int b,int c,int d,uchar tp,uchar ar,uchar sp,
	     int bd,int sd,int bg,int hl,int as,int ab,int ah)
  :FIGLET(w,a,b,c,d)
{
  colors = new int[7];
  if (!colors) { window = NULL; return; }
  pen = window->dpen();
  set = window->cset();
  type = tp;
  arrow = ar;
  space = sp;
  colors[0]=bd;
  colors[1]=sd;
  colors[2]=bg;
  colors[3]=hl;
  colors[4]=as;
  colors[5]=ab;
  colors[6]=ah;
  window->Add(this);
  window->Activate(this);
}

ARROW::~ARROW(void)
{
  if (!window) return;            // Nothing to do, something wrong with figlet
  if (colors) delete [] colors;       // If there are colorindexes, delete them
  window->DeActivate(this);                               // Not active anymore
  window->Del(this);                                    // Not anything anymore
}

void ARROW::Refresh(void)
{
  int a=xa,b=ya,c=xb,d=yb;
  if (type & arFRAME) {                                           // Draw Frame
    pen->Foreground((*set)[colors ? colors[0] : ocBORDER]);
    pen->Box(a,b,c,d);
    ++a; ++b; --c; --d;
  }
  if (!(type & arNOBACK)) {
    a+=type & 0x0F;
    b+=type & 0x0F;
    c-=type & 0x0F;
    d-=type & 0x0F;
    pen->Foreground((*set)[colors ? colors[2] : ocBACKGROUND]);
    pen->Fbox(a,b,c,d);
  }
  Draw();
}

void ARROW::Draw(void)
{
  int a=xa,b=ya,c=xb,d=yb;
  if (type & arFRAME) {
    ++a; ++b; --c; --d;
  }
  if (type & 0x0F) {
    if (type & arSTABLE)
      pen->Box3d(a,b,c,d,type & 0x0F,(*set)[colors ? colors[1] : ocSHADOW],
		 (*set)[colors ? colors[3] : ocHIGHLIGHT]);
    else {
      if (type & arIN)
	pen->Box3d(a,b,c,d,type & 0x0F,(*set)[colors? colors[3]: ocHIGHLIGHT],
		   (*set)[colors ? colors[1] : ocSHADOW]);
      else
	pen->Box3d(a,b,c,d,type & 0x0F,(*set)[colors ? colors[1] : ocSHADOW],
		   (*set)[colors ? colors[3] : ocHIGHLIGHT]);
    }
    a+=type & 0x0F; b+=type & 0x0F; c-=type & 0x0F; d-=type & 0x0F;
  }
  a += space & 0x3F;
  b += space & 0x3F;
  c -= space & 0x3F;
  d -= space & 0x3F;
  
  if ((space & arSTABLE) || (space & arIN))
    pen->Arrow(a,b,c,d,arrow,
	       (*set)[colors ? colors[6] : ocHIGHLIGHT],
	       (*set)[colors ? colors[6] : ocHIGHLIGHT],
	       (*set)[colors ? colors[4] : ocSHADOW],
	       (*set)[colors ? colors[5] : ocBACKGROUND]);
  else 
    pen->Arrow(a,b,c,d,arrow,
	       (*set)[colors ? colors[4] : ocSHADOW],
	       (*set)[colors ? colors[4] : ocSHADOW],
	       (*set)[colors ? colors[6] : ocHIGHLIGHT],
	       (*set)[colors ? colors[5] : ocBACKGROUND]);
}

char ARROW::Press(uchar,int,int)
{
  if (!(type & arSTABLE)) type ^= arIN;
  if (!(space & arSTABLE)) space ^= arIN; 
  Draw();
  return 1;
}

char ARROW::Motion(uchar,int x,int y)
{
  if (!Inside(x,y)) {
    if (!(type & arSTABLE)) type ^= arIN;
    if (!(space & arSTABLE)) space ^= arIN; 
    Draw();
    return 0;
  }
  return 1;
}

char ARROW::Release(uchar,int,int)
{
  if (!(type & arSTABLE)) type ^= arIN;
  if (!(space & arSTABLE)) space ^= arIN; 
  Draw();
  return 0;
}
