/*
   NAME:	button.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 15 1996
   COMMENT:	BUTTON
*/

#include "button.h"

BUTTON::BUTTON(WINDOW *w,int a,int b,int c,int d,char *tt,
	       uchar tp,uchar wd,uchar tr,ELEMENT *to)
  : FIGLET(w,a,b,c,d),
    pressed(0),
    title(NULL)
{
  // Set title of button, button will have no title
  Title(tt);
  // Set button settings
  cset = window->cset();
  pen = window->dpen();
  type = tp;
  thick = wd;
  trim = tr;
  trigger_element = to;
  col[0] = ocBORDER;
  col[1] = ocSHADOW;
  col[2] = ocBACKGROUND;
  col[3] = ocHIGHLIGHT;
  col[4] = ocFOREGROUND;
  col[5] = ocDARK;
  col[6] = ocLIGHT;
  callback = NULL;
  // Activate this figlet
  window->Add(this);
  window->Activate(this);
}

BUTTON::BUTTON(WINDOW *w,COLORSET *cs,int a,int b,int c,int d,char *tt,
	       uchar tp,uchar wd,uchar tr,ELEMENT *to)
  :FIGLET(w,a,b,c,d),
   pressed(0)
{
  // Set title of button, button will have no title
  title = NULL;
  Title(tt);
  // Set button settings
  cset = cs ? cs : window->cset();
  pen = window->dpen();
  type = tp;
  thick = wd;
  trim = tr;
  callback = NULL;
  trigger_element = to;
  col[0] = ocBORDER;
  col[1] = ocSHADOW;
  col[2] = ocBACKGROUND;
  col[3] = ocHIGHLIGHT;
  col[4] = ocFOREGROUND;
  col[5] = ocDARK;
  col[6] = ocLIGHT;
  // Activate this figlet
  window->Add(this);
  window->Activate(this);
}

BUTTON::BUTTON(WINDOW *w,COLORSET *cs,PEN *pn,
	       int a,int b,int c,int d,char *tt,
	       uchar tp,uchar wd,uchar tr,ELEMENT *to,
	       int c1,int c2,int c3,int c4,int c5,int c6,int c7)
  :FIGLET(w,a,b,c,d),
   pressed(0)
{
  //IBUG("new BUTTON(\"%s\")%c%c",tt,0,0);
  // Set title of button, button will have no title
  title = NULL;
  Title(tt);
  // Set button settings
  cset = cs ? cs : window->cset();
  type = tp;
  thick = wd;
  trim = tr;
  callback = NULL;
  trigger_element = to;
  pen = pn ? pn : window->dpen();
  // Set colors, if it fails, default colorindexes will be used
  col[0] = c1;
  col[1] = c2;
  col[2] = c3;
  col[3] = c4;
  col[4] = c5;
  col[5] = c6;
  col[6] = c7;
  // Activate this figlet
  window->Add(this);
  window->Activate(this);
  //OBUG("new BUTTON(\"%s\")%c%c",tt,0,0);
}

BUTTON::~BUTTON(void)
{
  // Deactivate button
  window->DeActivate(this);
  window->Del(this);
  // Free titlememory
  if (title) delete [] title;
}

const char *BUTTON::Title(char *tt)
  /// If tt is NULL, the titles string is returned
  /// Else tt is attempted copied into the title, NULL returned if it failed
{
  if (!tt) return (const char*)title;
  if (title) {
    char *t = new char[strlen(tt)+1];
    if (!t) {
      WARN("BUTTON[%p].Title(\"%s\") could not allocate titlememory",
	   this,tt);
      return NULL;
    }
    strcpy(t,tt);
    delete [] title;
    title = t;
  } else {
    title = new char[strlen(tt)+1];
    if (!title) return NULL;
    strcpy(title,tt);    
  }
  return title;
}

void BUTTON::Refresh(void)
  /// Notice that drawing now has it's own function, that only draws what
  /// changes within the button, this makes quicker drawing in X11.
{
  int a = xa,b = ya,c = xb,d = yb;
  if (type & butFRAME) {
    pen->Foreground((*cset)[col[0]]);
    pen->Box(a,b,c,d);
    ++a; b++; --c; --d;    
  }
  if (type & butBORDER) {
    if (type & butSUNKEN)
      pen->Fbox3d(a,b,c,d,(type&0x7)+1,(*cset)[col[1]],(*cset)[col[2]],
		  (*cset)[col[3]]);
    else
      pen->Fbox3d(a,b,c,d,(type&0x7)+1,(*cset)[col[3]],(*cset)[col[2]],
		  (*cset)[col[1]]);
  } else {
    pen->Foreground((*cset)[col[2]]);
    pen->Fbox(a,b,c,d);
  }
  Draw();
}

void BUTTON::Draw(void)
{
  int a = xa,b = ya,c = xb,d = yb;
  if (type & butFRAME) { ++a; b++; --c; --d; }
  if (type & butBORDER) {
    a+=((type&0x7)+1)*2; b+=((type&0x7)+1)*2;
    c-=((type&0x7)+1)*2; d-=((type&0x7)+1)*2;
  }
  if (type & butIN) 
    pen->Box3d(a,b,c,d,thick & 0x3F,(*cset)[col[3]],(*cset)[col[1]]);
  else
    pen->Box3d(a,b,c,d,thick & 0x3F,(*cset)[col[1]],(*cset)[col[3]]);
  if (title) {
    int x,y;
    pen->Justify(&x,&y,xa+(thick & 0x3F),ya+(thick & 0x3F),
		 xb-(thick & 0x3F),yb-(thick & 0x3F),trim,title);
    if (type & butIN) {
      if ((thick & but3DTEXT) == but3DTEXT)
	pen->Write3d(x,y,title,(*cset)[col[3]],(*cset)[col[4]],
		     (*cset)[col[1]]);
      else {
	if (((thick & butMOVETEXT) == butMOVETEXT) ||
	    ((thick & butMOVEDARK) == butMOVEDARK)) {
	  pen->Foreground((*cset)[col[2]]);
	  pen->Write(x,y,title);
	}

	if ((thick & butMOVETEXT) == butMOVETEXT)
	  pen->Foreground((*cset)[col[4]]);
	else
	  pen->Foreground((*cset)[col[5]]);

	if (((thick & butMOVETEXT) == butMOVETEXT) ||
	    ((thick & butMOVEDARK) == butMOVEDARK))
	  pen->Write(x-1,y-1,title); 
	else
	  pen->Write(x,y,title);
     }


    } else {
      if ((thick & but3DTEXT) == but3DTEXT)
	pen->Write3d(x,y,title,(*cset)[col[1]],(*cset)[col[4]],
		     (*cset)[col[3]]);
      else {
	if (((thick & butMOVETEXT) == butMOVETEXT) ||
	    ((thick & butMOVEDARK) == butMOVEDARK)) {
	  pen->Foreground((*cset)[col[2]]);
	  pen->Write(x-1,y-1,title);
	}
	
	if ((thick & butMOVETEXT) == butMOVETEXT)
	  pen->Foreground((*cset)[col[4]]);
	else
	  pen->Foreground((*cset)[col[6]]);

	pen->Write(x,y,title);
      }
    }
  }
}

char BUTTON::Press(uchar,int,int)
{
  pressed = 1;
  type ^= butIN;
  Draw();
  return 1;
}

char BUTTON::Motion(uchar,int x,int y)
{
  if (!Inside(x,y)) {
    pressed = 0;
    type ^= butIN;
    Draw();
    return 0;
  }
  return 1;
}

char BUTTON::Release(uchar,int,int)
{
  if (!pressed) return 0;
  switch (type & butHOLD) {
  case butCLICK:
    type ^= butIN;
    Draw();
  case butHOLD:
    if (callback)
      callback(this);
    else {
      if (trigger_element)
	trigger_element->Trigger(this);
      else
	Trigger(this);
    }
    break;
  }
  pressed = 0;
  return 0;
}

char BUTTON::Pressed(void)
{
  return (type & butIN);
}

void BUTTON::State(uchar st)
{
  if (st & butIN)
    type |= butIN;
  else
    type &= ~butIN;
  Draw();
}

void BUTTON::Callback(char(*cb)(BUTTON*))
{
  callback = cb;
}

void BUTTON::Element(ELEMENT *e)
{
  trigger_element = e;
}
