/*
   NAME:	button.h
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 15 1996
   COMMENT:	BUTTON figlet for buttons
*/
#ifndef BUTTONH
#define BUTTONH

#include "otools.h"

// BUTTON type IDENTIFIERS | 0-7
#define butCLICK   0x00
#define butHOLD    0x10
#define butIN      0x20
#define butSUNKEN  0x40
#define butFRAME   0x80
#define butBORDER  0x08

// BUTTON thickness IDENTIFIERS | 0-63
#define butDARKTEXT 0x00
#define butMOVETEXT 0x40
#define butMOVEDARK 0x80
#define but3DTEXT   0xC0

class BUTTON:public FIGLET {
  /// Implementation of visual buttons, the ordinary clickable ones and also
  /// buttons that stay clicked. There are two methods to get information
  /// wether a button was activated (that is, pressed then released).
protected:
  PEN *pen;
  COLORSET *cset;                                                   // Colorset
  uchar trim,type,thick,pressed;                          // Look of the button
  ELEMENT *trigger_element;        // An elements Trigger() to call when pushed
  char *title;                                // The buttons title, single line
  int col[7];           // Contains colors used for buttons (bd,sd,bg,hl,dk,lt)
  char (*callback)(BUTTON*);
public:

  BUTTON(WINDOW*,int,int,int,int,char*,uchar,uchar,uchar,ELEMENT*);
  //. The window to draw the button in with standard colors,
  //. The size of the button relative to the window,
  //. The title of the button,
  //. buttontype: (butCLICK/HOLD | FRAME|BORDER|SUNKEN | IN | 0-7),
  //. 3d thickness, (butDARKTEXT/MOVETEXT/MOVEDARK/3DTEXT|0-63),
  //. The TRIM (oLEFT/RIGHT/MIDDLE/HWIDE/TOP/BOTTOM/VWIDE|0-15),
  //. Trigger() element (won't trig if it's NULL).

  BUTTON(WINDOW*,     // The window to draw the button in, with standard colors
	 COLORSET*,                      // The colorset to use for this button
	 int,int,int,int,      // The size of the button relative to the window
	 char*,                                      // The title of the button
	 uchar, // buttontype: (butCLICK/HOLD | FRAME|BORDER|SUNKEN | IN | 0-7)
	 uchar,    // 3d thickness, (butDARKTEXT/MOVETEXT/MOVEDARK/3DTEXT|0-63)
	 uchar,    // The TRIM (oLEFT/RIGHT/MIDDLE/HWIDE/TOP/BOTTOM/VWIDE|0-15)
	 ELEMENT*);                // Trigger() element, NULL -> this.Trigger()

  BUTTON(WINDOW*,                 // The window to draw the button in, advanced
	 COLORSET*,                      // The colorset to use for this button
	 PEN*,          // The pen to use (must have same colormap as colorset)
	 int,int,int,int,      // The size of the button relative to the window
	 char*,                                      // The title of the button
	 uchar, // buttontype: (butCLICK/HOLD | FRAME|BORDER|SUNKEN | IN | 0-7)
	 uchar,    // 3d thickness, (butDARKTEXT/MOVETEXT/MOVEDARK/3DTEXT|0-63)
	 uchar,    // The TRIM (oLEFT/RIGHT/MIDDLE/HWIDE/TOP/BOTTOM/VWIDE|0-15)
	 ELEMENT*,                 // Trigger() element, NULL -> this.Trigger()
	 int,int,int,int,int,int,int);   // Specified colors: bd,sd,bg,hl,dk,lt

  virtual ~BUTTON(void);
  virtual const char *Title(char*);
  //. parameter = NULL will return title string, else set the title of the
  //. button.
  virtual void Refresh(void);
  virtual void Draw(void);
  virtual char Press(uchar,int,int);
  virtual char Motion(uchar,int,int);
  virtual char Release(uchar,int,int);
  virtual char Pressed(void);
  virtual void State(uchar);
  virtual void Callback(char(*)(BUTTON*));
  virtual void Element(ELEMENT*);
};

#endif /* BUTTONH */

