/*
   NAME:        debug.h
   AUTHOR:      Ole Gunnar Westgaard TFU <olew@benoni>
   DATE:        Jul 17 1997
   COMMENT:     Debugging functionality
*/
#ifndef DEBUGH
#define DEBUGH

#ifdef DEBUG_OTOOLS

#include <string.h>

extern unsigned long Dl;
extern char D[];

#define D_IBUGOBUG  0x00000001
#define D_DBUG      0x00000002

#define D_OTOOLS    0x00000004
#define D_DISPLAY   0x00000008
#define D_COLORMAP  0x00000010
#define D_COLORSET  0x00000020
#define D_WINDOW    0x00000040
#define D_PEN       0x00000080
#define D_FIGLET    0x00000100

#define D_FRAME     0x00000200
#define D_BUTTON    0x00000400
#define D_SWITCH    0x00000800
#define D_INPUT     0x00001000
#define D_SLIDER    0x00002000
#define D_ARROW     0x00004000
#define D_TEXT      0x00008000
#define D_SELECTOR  0x00010000
#define D_MENU      0x00020000
#define D_FILES     0x00040000
#define D_OPENGL    0x00080000
#define D_ITEM      0x00100000
#define D_SUBMENU   0x00200000
#define D_LIST      0x00400000

#define D_GARBAGE   0x10000000
#define D_LOW       0x20000000
#define D_MED       0x40000000
#define D_HIGH      0x80000000

#define D_ALL       0x0FFFFFFC

#define IBUG(code,format,args...) {\
  if ((Dl & D_IBUGOBUG) && ((Dl | code) == Dl)) {\
    fprintf(stderr,"%s\n%s/ ",D,D);\
    fprintf(stderr,format,##args);\
    fprintf(stderr,"\n");\
    strcat(D,"| ");\
  }\
 }

#define OBUG(code,format,args...) {\
  if ((Dl & D_IBUGOBUG) && ((Dl | code) == Dl)) {\
    D[strlen(D)-2]='\0';\
    fprintf(stderr,"%s\\ ",D);\
    fprintf(stderr,format,##args);\
    fprintf(stderr,"\n");\
  }\
 }

#define DBUG(code,format,args...) {\
  if ((Dl & D_DBUG) && ((Dl | code) == Dl)) {\
    fprintf(stderr,"%s ",D);\
    fprintf(stderr,format,##args);\
    fprintf(stderr,"\n");\
  }\
 }

#define WARN(format,args...) {\
  fprintf(stderr,"%s Warning: ",D);\
  fprintf(stderr,format,##args);\
  fprintf(stderr,"\n");\
 }

#else

#define IBUG(code,format,args...)
#define OBUG(code,format,args...)
#define DBUG(code,foramt,args...)
#define WARN(format,args...) {\
  fprintf(stderr,"Warning: ");\
  fprintf(stderr,format,##args);\
  fprintf(stderr,"\n");\
 }

#endif

#define EXIT(format,args...) {\
  fprintf(stderr,format,##args);\
  fprintf(stderr,"\n");\
  exit(0);\
 }

#endif /* DEBUGH */
