/*
   NAME:	Frame.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 12 1996
   COMMENT:	FRAME figlet for drawing frames
*/

#include "frame.h"

FRAME::FRAME(WINDOW *wnd,int a,int b,int c,int d,uchar l,uchar t,int index)
  :FIGLET(wnd),
   figlets(5,5)
{
  IBUG(D_FRAME|D_MED,"new FRAME[%p](WINDOW[%p],(light)...)",this,wnd);
  look = l; thick = t;
  bd=index; sd =bd+1; bg=sd+1; hl=bg+1; dk=hl+2;
  pen = window->dpen();
  cset = window->cset();
  method = 0; we = he = cx = cy = 0;
  Resize(a,b,c,d);
  window->Add(this);
  OBUG(D_FRAME|D_MED,"Created.");
}

FRAME::FRAME(WINDOW *wnd,COLORSET *clr,
	     int a,int b,int c,int d,
	     uchar l,uchar t,
	     int c1,int c2,int c3,int c4,int c5)
  :FIGLET(wnd),
   figlets(5,5)
{
  IBUG(D_FRAME|D_MED,"new FRAME[%p](WINDOW[%p],(heavy)...)",this,wnd);
  look = l; thick = t; bd = c1; sd = c2; bg = c3; hl = c4; dk = c5;
  pen = window->dpen();
  cset = clr; method = 0; space = 0;
  we = he = cx = cy = 0;
  Resize(a,b,c,d);
  window->Add(this);
  OBUG(D_FRAME|D_MED,"Created.");
}

FRAME::~FRAME(void)
  /// Removes a frame, gives back all figlet to their windows refresh chains.
{
  IBUG(D_FRAME|D_MED,"~FRAME[%p]()",this);
  FIGLET *fig;
  while ((fig = (FIGLET*)figlets[1])) {
    figlets.remove(fig);
    fig->wnd()->Add(fig);
  }
  window->Del(this);
  OBUG(D_FRAME|D_MED,"Deleted.");
}

char FRAME::Resize(int a,int b,int c,int d)
{
  FIGLET::Resize(a,b,c,d);
  // Calculate inside of figlet
  ixa = xa; iya = ya; ixb = xb; iyb = yb;
  switch (look & 0x70) {
  case bTGL:
  case bDBL: ixa+=3; iya+=3; ixb-=3; iyb-=3;  break;
  case bSNG: ixa+=1; iya+=1; ixb-=1; iyb-=1; break;
  case bSHD: ixa+=1; iya+=1; ixb-=4; iyb-=4;  break;
  case bTHK: ixa+=2; iya+=2; ixb-=2; iyb-=2;  break;
  case bTWO: ixa+=3; iya+=3; ixb-=3; iyb-=3;  break;
  case bTSD: ixa+=4; iya+=4; ixb-=1; iyb-=1;  break;
  }
  switch (look & 0x07) { // frame
  case fTRP: ixa+=thick;   iya+=thick;   ixb-=thick;   iyb-=thick;
  case fDBL: ixa+=thick;   iya+=thick;   ixb-=thick;   iyb-=thick;
  case fSNG: ixa+=thick;   iya+=thick;   ixb-=thick;   iyb-=thick;   break;
  case fWID: ixa+=thick+4; iya+=thick+4; ixb-=thick+4; iyb-=thick+4; break;
  case fPLT: ixa+=2;       iya+=2;       ixb-=2;       iyb-=2;       break;
  case fSNK: ixa+=thick+2; iya+=thick+2; ixb-=thick+2; iyb-=thick+2; break;
  case fFLR: ixa+=thick*2; iya+=thick*2; ixb-=thick*2; iyb-=thick*2; break;
  }
  // printf("Frame size %d,%d  %d,%d\n",ixa,iya,ixb,iyb);

  // Width and height of figlet, not counting spaces between figlets
  if (we) width  = ((float)(ixb-ixa+1-space*2))/((float)(we));
  if (he) height = ((float)(iyb-iya+1-space*2))/((float)(he));
  return 1;
}

void FRAME::Refresh(void)
{
  ulong l,d;

  /*
  // border
  switch (look & 0x70) {
  case bSNG:
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[bd]);
    pen->Box(XA,YA,XB,YB);
    XA+=1; YA+=1; XB-=1; YB-=1;
    break;
  case bDBL:
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[bd]);
    pen->Box(XA,YA,XB,YB);
    pen->Box(XA+2,YA+2,XB-2,YB-2);
    XA+=3; YA+=3; XB-=3; YB-=3;
    break;
  case bSHD:
    pen->Foreground((*cset)[bd]);
    pen->Box(XA,YA,XB-3,YB-3);
    if (look & 0x80) pen->Foreground((*cset)[sd]);
    pen->Fbox(XA+3,YB-2,XB,YB);
    pen->Fbox(XB-2,YA+3,XB,YB-3);
    XA+=1; YA+=1; XB-=4; YB-=4;
    break;
  case bTGL:
    pen->Foreground((*cset)[bd]);
    if (look & 0x80) {
      pen->Box(XA+2,YA,XB,YB-2);
      pen->Box(XA,YA+2,XB-2,YB);
    } else {
      pen->Box(XA,YA,XB-2,YB-2);
      pen->Box(XA+2,YA+2,XB,YB);
    }
    XA+=3; YA+=3; XB-=3; YB-=3;
    break;
  case bTHK:
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[bd]);
    pen->Box(XA,YA,XB,YB);
    pen->Box(XA+1,YA+1,XB-1,YB-1);
    XA+=2; YA+=2; XB-=2; YB-=2;
    break;
  case bTWO:
    pen->Foreground((look & 0x80) ? (*cset)[hl] : (*cset)[sd]);
    pen->Box(XA,YA,XB-2,YB-2);
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[hl]);
    pen->Box(XA+2,YA+2,XB,YB);
    XA+=3; YA+=3; XB-=3; YB-=3;
    break;
  case bTSD:
    pen->Foreground((*cset)[bd]);
    pen->Box(XA+3,YA+3,XB,YB);
    if (look & 0x80) pen->Foreground((*cset)[sd]);
    pen->Fbox(XA,YA,XB-3,YA+2);
    pen->Fbox(XA,YA+2,XA+2,YB-3);
    XA+=4; YA+=4; XB-=1; YB-=1;
    break;
  }
  */
  // border
  switch (look & 0x70) {
  case bSNG:
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[bd]);
    pen->Box(xa,ya,xb,yb);
    break;
  case bDBL:
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[bd]);
    pen->Box(xa,ya,xb,yb);
    pen->Box(xa+2,ya+2,xb-2,yb-2);
    break;
  case bSHD:
    pen->Foreground((*cset)[bd]);
    pen->Box(xa,ya,xb-3,yb-3);
    if (look & 0x80) pen->Foreground((*cset)[sd]);
    pen->Fbox(xa+3,yb-2,xb,yb);
    pen->Fbox(xb-2,ya+3,xb,yb-3);
    break;
  case bTGL:
    pen->Foreground((*cset)[bd]);
    if (look & 0x80) {
      pen->Box(xa+2,ya,xb,yb-2);
      pen->Box(xa,ya+2,xb-2,yb);
    } else {
      pen->Box(xa,ya,xb-2,yb-2);
      pen->Box(xa+2,ya+2,xb,yb);
    }
    break;
  case bTHK:
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[bd]);
    pen->Box(xa,ya,xb,yb);
    pen->Box(xa+1,ya+1,xb-1,yb-1);
    break;
  case bTWO:
    pen->Foreground((look & 0x80) ? (*cset)[hl] : (*cset)[sd]);
    pen->Box(xa,ya,xb-2,yb-2);
    pen->Foreground((look & 0x80) ? (*cset)[sd] : (*cset)[hl]);
    pen->Box(xa+2,ya+2,xb,yb);
    break;
  case bTSD:
    pen->Foreground((*cset)[bd]);
    pen->Box(xa+3,ya+3,xb,yb);
    if (look & 0x80) pen->Foreground((*cset)[sd]);
    pen->Fbox(xa,ya,xb-3,ya+2);
    pen->Fbox(xa,ya+2,xa+2,yb-3);
    break;
  }

  if (look & 0x08) {
    l = (*cset)[hl];
    d = (*cset)[sd]; 
  } else {
    d = (*cset)[hl];
    l = (*cset)[sd]; 
  }

  switch (look & 0x07) { // frame
  case fTRP:
    pen->Box3d(ixa-thick*3,iya-thick*3,ixb+thick*3,iyb+thick*3,thick,l,d);
  case fDBL:
    pen->Box3d(ixa-thick*2,iya-thick*2,ixb+thick*2,iyb+thick*2,thick,d,l);
  case fSNG:
    pen->Box3d(ixa-thick,iya-thick,ixb+thick,iyb+thick,thick,l,d);
    break;
  case fWID:
    pen->Box3d(ixa-2,iya-2,ixb+2,iyb+2,2,d,l);
    pen->Box3d(ixa-thick-2,iya-thick-2,ixb+thick+2,iyb+thick+2,thick,l,d);
    pen->Box3d(ixa-thick-4,iya-thick-4,ixb+thick+4,iyb+thick+4,2,d,l);
    break;
  case fPLT:
    pen->Box3d(ixa-2,iya-2,ixb+2,iyb+2,2,l,d);
    break;
  case fSNK:
    pen->Box3d(ixa-2,iya-2,ixb+2,iyb+2,2,l,d);
    pen->Box3d(ixa-thick-4,iya-thick-4,ixb+thick+4,iyb+thick+4,2,d,l);
    break;
  case fFLR:
    pen->Box3d(ixa-thick,iya-thick,ixb+thick,iyb+thick,thick,l,d);
    pen->Box3d(ixa-thick*2-3,iya-thick*2-3,ixb+thick*2+3,iyb+thick*2+3,
	       thick,d,l);
    break;
  }
  pen->Foreground((*cset)[bg]);
  pen->Fbox(ixa,iya,ixb,iyb);

  /*  switch (look & 0x07) { // frame
  case fTRP:
    pen->Box3d(XA+thick*2,YA+thick*2,XB-thick*2,YB-thick*2,thick,d,l);
  case fDBL:
    pen->Box3d(XA+thick,YA+thick,XB-thick,YB-thick,thick,d,l);
  case fSNG:
    pen->Box3d(XA,YA,XB,YB,thick,l,d);
    break;
  case fWID:
    pen->Box3d(XA     ,YA     ,XB     ,YB     ,2 ,d,l);
    pen->Box3d(XA+2   ,YA+2   ,XB-2   ,YB-2   ,thick,l,d);
    pen->Box3d(XA+2+thick,YA+2+thick,XB-2-thick,YB-2-thick,2 ,d,l);
    break;
  case fPLT:
    pen->Box3d(XA,YA,XB,YB,2,l,d);
    break;
  case fSNK:
    pen->Box3d(XA,YA,XB,YB,2,l,d);
    pen->Box3d(XA+2+thick,YA+2+thick,XB-2-thick,YB-2-thick,2,d,l);
    break;
  case fFLR:
    pen->Box3d(XA,YA,XB,YB,thick,l,d);
    pen->Box3d(XA+thick+3,YA+thick+3,XB-thick-3,YB-thick-3,thick,d,l);
    break;
  }
  */
  { // While refreshing, draw chained figlets
    // printf("Figlets: %d\n",figlets.number_of_elements());
    FIGLET *fig;
    for (ulong i=1; (fig = (FIGLET*)figlets[i]); ++i) fig->Refresh();
  }
}

char FRAME::Add(FIGLET *figlet)
  /// Adds the figlet to the frames redraw chain, releases it from the
  /// Windows Refresh chain.
{
  if (figlets.append(figlet)) {
    figlet->wnd()->Del(figlet);
    StackSingle(figlet);
    return 1;
  }
  return 0;
}

char FRAME::Del(FIGLET *figlet)
  /// Removes the figlet from the frames redraw chain, gives it back
  /// to the figlets windows refresh chain.
{
  if (figlets.remove(figlet)) {
    figlet->wnd()->Add(figlet);
    Refresh();
    return 1;
  }
  return 0;
}  

void FRAME::Stacking(int w,int h,int x,int y,uchar m,uchar s)
{
  we = w; he = h; cx = x; cy = y; method = m; space = s;
  Resize(xa,ya,xb,yb);
}

void FRAME::Stack(void)
{
}

void FRAME::StackSingle(FIGLET *f)
  /// Stacks a single figlet into the frame, will not do this if there is no
  /// space for the figlet.
{
  int s = method & 0x1F;
  // Don't stack figlet if there are no method to stack them with
  if ((we==0) || (he==0)) return;

  // Don't stack if stackposition is outside the stack
  if ((cx<1) || (cx>we) || (cy<1) || (cy>he)) return;

  // Calculate the size of the stack(frame)window
  f->Resize(cx<2  ? ixa+space: ixa+space+(int)(width*(float)(cx-1)) + s/2,
	    cy<2  ? iya+space: iya+space+(int)(height*(float)(cy-1)) + s/2,
	    cx>we ? ixb-space+1: ixa+space+(int)(width*(float)(cx))-1 -(s+1)/2,
	    cy>he ? iyb-space+1: iya+space+(int)(height*(float)(cy))-1 -(s+1)/2
	    );
  /*
  a = a + (int)(width*(float)(cx-1)) + (cx>1 ? between/2 :0);
  b = b + (int)(height*(float)(cy-1)) + (cy>1 ? between/2 :0);
  c = a + (int)(width*(float)(cx)) + (cx<we ? (between+1)/2 :0);
  d = b + (int)(height*(float)(cy)) + (cy<he ? (between+1)/2 :0);
  */

  int a = method & frLEFT ? -1 : 1, b = method & frUP   ? -1 : 1;

  if (method & frVERFIRST) {
    cy+=b;
    if (cy>he) {
      cy=1;
      cx+=a;
      if (cx>we) cx=1;
    }
  } else {
    cx+=a;
    if (cx>we) {
      cx=1;
      cy+=b;
      if (cy>he) cy=1;
    }
  }
}

void FRAME::Empty(void)
{
  FIGLET *fig;
  while ((fig = (FIGLET*)figlets[1])) {
    figlets.remove(fig);
    fig->wnd()->Add(fig);
  }
}
