/*
   NAME:	list.h
   AUTHOR:	Ole Gunnar Westgaard <olew@ifi.uio.no>
   DATE:	June 2 1996
   COMMENT:     A list class, to keep lists of elements
                Major changes to list class December 4 '96
*/

#ifndef LISTH
#define LISTH

#include "debug.h"

class ELEMENT {
  /// This is the class that the class LIST uses as atoms for its arrays.
  /// All objects that should be used with LIST must be derived from
  /// this class.
  ///
  /// The class contains a Trigger, this function can be used for various
  /// purposes. class OTOOLS uses this for executing timed functions.
public:
  inline virtual ~ELEMENT(void);
  //. Virtual destructor, gotta have this one
   inline virtual char Trigger(ELEMENT*);
  //. Userdefinable trigger, usually element reflects the elemnt that called
  //. the trigger.
  //> element->Trigger(this);
  //> Calls the Triggerfunction for element, telling that this is the caller.
};

class LIST : public ELEMENT {
  /// This class uses objects of type class ELEMENT in it's list
  /// implementation. Remember that the first element has index 1
  /// in this class!!!
public:
  inline                 LIST(unsigned long,unsigned short);
  //. Create a list with space for a number of elements and size of increment
  //. when the list need resizing.
  inline                ~LIST(void);
  //. Remove this list.
  inline unsigned long   append(ELEMENT*);
  //. Append element. The element will be given a key=(prev.key+1).
  inline unsigned long   add(ELEMENT*,unsigned long);
  //. Add element with key.
  inline unsigned long   remove(ELEMENT*);
  //. Delete element.
  inline unsigned long   del(unsigned long);
  //. Delete element with key.
  inline unsigned long   locate(unsigned long);
  //. find elementindex with key
  inline ELEMENT        *operator[](unsigned long);
  //. get element at index
  inline ELEMENT        *find(unsigned long);
  //. find element with key
  inline unsigned long   get_index(ELEMENT*);
  //. find index of element
  inline unsigned long   key(unsigned long);
  //. get key of element at index
  inline unsigned long   key(ELEMENT*);
  //. get key of element
  inline unsigned long   number_of_elements(void);
  //. get number of elements
private:
  ELEMENT              **elements;                             // element array
  unsigned long         *keys,count,size; // keys,number of elements,array size
  unsigned short         increment;        // how much to enlarge when resizing
  inline char            resize_up(void);     // enlarge if neccesary 0 = error
};

#include "list.i"

#endif /* LISTH */

