/*
   NAME:        list.i
   AUTHOR:      root <root@>
   DATE:        Dec  8 1996
   COMMENT:     Inline functions of LIST
*/

#ifndef LISTI
#define LISTI

#include <stdio.h>

inline ELEMENT::~ELEMENT(void)
{
}

inline char ELEMENT::Trigger(ELEMENT*)
{
  return 0;  
}

inline LIST::LIST(unsigned long set_size,unsigned short set_increment)
{
  /// Creates an array representing the list. the number of elements in
  /// the list is zero. does not check wether new returns NULL.
  size = (set_size < 5 ? 5 : set_size);
  increment = (set_increment < 3 ? 3 : set_increment);
  count = 0;
  elements = new ELEMENT*[size];
  keys = new unsigned long[size];
}

inline LIST::~LIST(void)
{
  /// Destroys the list class.
  /// Deletes the allocated array, freeing the memory eaten by it.
  if (elements) delete [] elements;
  if (keys) delete [] keys;
}

inline unsigned long LIST::append(ELEMENT *element)
{
  /// Append an element to the end of this list, set the key to be the value
  /// of the previous elements key.
  if (!resize_up()) return 0;          // 0 means that it could not be appended
  elements[count] = element;
  if (count > 0)
    keys[count] = keys[count-1] + 1;               // previous elements key + 1
  else
    keys[count] = 1;                            // first element, we start at 1
  ++count;
  return count;             // return index of element (starting at 1 remember)
}

inline unsigned long LIST::add(ELEMENT *element,unsigned long key)
{
  IBUG(D_LIST|D_LOW,"LIST[%p].add(%p,%ld)",this,element,key);
  /// Add an element into a sorted array of keys.
  /// Duplicate elements are also added, next to each other.
  if (!resize_up()) return 0;             // 0 means that it could not be added
  unsigned long i = 0;
  if (count > 0) {                // If there are elements in the list allready
    while ((i<count) && (keys[i]<=key)) ++i;
    for (unsigned long j = count-1; j >= i; --j) {              // Restack list
      elements[j+1] = elements[j];
      keys[j+1] = keys[j];
    }
  }
  elements[i] = element;                               // Set element into list
  keys[i] = key;
  ++count;
  OBUG(D_LIST|D_LOW,"Added element in sorted array");
  return i+1;          // return index of added element, starting at 1 remember
}

inline unsigned long LIST::remove(ELEMENT *element)
{
  /// Finds and removes the element from the list.
  if (count == 0) return 0;
  unsigned long i = get_index(element);           // find element of discussion
  if (i == 0) return 0;                       // element does not exist in list
  for (unsigned long j = i; j < count; ++j) {                   // restack list
    elements[j-1] = elements[j];
    keys[j-1] = keys[j];
  }
  --count;                                          // one less element in list
  return i;                            // return the index that the element had
}

inline unsigned long LIST::del(unsigned long key)
{
  /// Finds and removes the element from the list.
  if (count == 0) return 0;
  unsigned long i = locate(key);                       // find element with key
  if (i == 0) return 0;                       // element does not exist in list
  for (unsigned long j = i; j < count; ++j) {                   // restack list
    elements[j-1] = elements[j];
    keys[j-1] = keys[j];
  }
  --count;                                          // one less element in list
  return i;                            // return the index that the element had
}

inline unsigned long LIST::locate(unsigned long key)
{
  /// Returns the position of an element with key, if the element does not
  /// exist, 0 will be returned. This function uses a binary search.
  IBUG(D_LIST|D_LOW,"LIST[%p].locate(key=%ld) count = %ld",this,key,count);
  unsigned long p=0, a = 0, b = count - 1;
  switch (count) {
  case 0:
    OBUG(D_LIST|D_LOW,"No items in list.");
    return 0;
    break;
  case 1:
    if (keys[0] == key) {
      OBUG(D_LIST|D_LOW,"Single item has correct key.");
      return p+1;
    } else {
      OBUG(D_LIST|D_LOW,"Single item has other key.");
      return 0;
    }
    break;
  default:
    // Check if the key is within the boundaries
    if ((keys[0]>key) || (keys[count-1]<key)) {
      OBUG(D_LIST|D_LOW,"Item not within set.");
      return 0;
    }
    // Binary search now that it lies wihtin
    while (a <= b) {
      p = (a + b) / 2;
      if (keys[p] == key) {
	OBUG(D_LIST|D_LOW,"Item found.");
	return p+1;
      } else {
	if (keys[p] > key) b = p-1; else a = p+1;
      }
    }
    // Not in the sorted list
    OBUG(D_LIST|D_LOW,"Item not found!");
    return 0;
    break;
  }
}

inline ELEMENT *LIST::operator[](unsigned long index)
{
  if ((index > count) || (index == 0)) return NULL;
  return elements[index-1];
}

inline ELEMENT *LIST::find(unsigned long key)
{
  unsigned long i = locate(key);
  if (i == 0) return NULL;
  return elements[i-1];
}

inline unsigned long LIST::key(unsigned long index)
{
  if ((index > count) || (index == 0)) return 0;
  return keys[index-1];
}

inline unsigned long LIST::key(ELEMENT *element)
{
  unsigned long i = get_index(element);
  if (i == 0) return 0;
  return keys[i-1];
}

inline unsigned long LIST::number_of_elements(void)
{
  return count;
}

inline unsigned long LIST::get_index(ELEMENT *element)
{
  /// Traverse list to find element, best method, because the list is not
  /// sorted by element, only by key.
  for (unsigned long i=0; i<count; ++i)                    // Traverse elements
    if (elements[i] == element) return i+1;            // return index if found
  return 0;                                       // Could not find the element
}

inline char LIST::resize_up(void)
{ 
  /// Checks wether list must be enlarged if another element is added.
  /// Will enlarge the list if neccesary.
  if ((count+1) < size) return 1;                               // large enough
  ELEMENT **el = new ELEMENT*[size+increment];
  unsigned long *kl = new unsigned long[size+increment];
  if ((el == NULL) || (kl == NULL)) {
    if (el) delete [] el;
    if (kl) delete [] kl;
    WARN("LIST[%p].resize_up() Not enough memory to enlarge list!",this);
    return 0;
  }
  for (unsigned long i=0; i<count; ++i) {
    el[i] = elements[i];
    kl[i] = keys[i];
  }
  size += increment;
  delete [] elements;
  delete [] keys;
  elements = el;
  keys = kl;
  return 1;
}

#endif /* LISTI */


