/*
   NAME:	menu.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Feb 12 1997
   COMMENT:	Implementation of a menu
*/

#include "menu.h"
#include "button.h"

#include <X11/cursorfont.h>

//=============================================================================

ITEM::~ITEM(void)
{
  // Delete ITEM and options / submenus following
  if (submenu) delete submenu;
  if (next) delete next;
  //DBUG("~ITEM(\"%s\") ok%c%c",name,0,0);
}

ITEM::ITEM(char *nm,void(*cb)(char*),ITEM *mu,ITEM *nx)
{
  // middle option w/callback
  IBUG(D_ITEM|D_LOW,"new ITEM[%p](\"%s\",cb[%p],submenu[%p],next[%p])",
       this,nm,cb,mu,nx);
  name     = nm;
  shortcut = -1;
  callback = cb;
  submenu  = mu;
  next     = nx;
  OBUG(D_ITEM|D_LOW,"Done.");
}

ITEM::ITEM(char *nm,void(*cb)(char*),ITEM *nx)
{
  // middle option w/callback
  IBUG(D_ITEM|D_LOW,"new ITEM[%p](\"%s\",cb[%p],next[%p])",this,nm,cb,nx);
  name     = nm;
  shortcut = -1;
  callback = cb;
  submenu  = NULL;
  next     = nx;
  OBUG(D_ITEM|D_LOW,"Done.");
}

ITEM::ITEM(char *nm,ITEM *mu,ITEM *nx)
{
  // middle option with submenu
  IBUG(D_ITEM|D_LOW,"new ITEM[%p](\"%s\",submenu[%p],next[%p])",
       this,nm,mu,nx);
  name     = nm;
  shortcut = -1;
  callback = NULL;
  submenu  = mu;
  next     = nx;
  OBUG(D_ITEM|D_LOW,"Done.");
}

ITEM::ITEM(char *nm,ITEM *nx)
{
  // middle option - disfunctional
  IBUG(D_ITEM|D_LOW,"new ITEM[%p](\"%s\",next[%p])",this,nm,nx);
  name     = nm;
  shortcut = -1;
  callback = NULL;
  submenu  = NULL;
  next     = nx;
  //  OBUG("ITEM(\"%s\",next)%c%c",name,0,0);
  OBUG(D_ITEM|D_LOW,"Done.");
}

ITEM::ITEM(char *nm,void(*cb)(char*))
{
  // end option with callback
  IBUG(D_ITEM|D_LOW,"new ITEM[%p](\"%s\",cb[%p])",this,nm,cb);
  name     = nm;
  shortcut = -1;
  callback = cb;
  submenu  = NULL;
  next     = NULL;
  OBUG(D_ITEM|D_LOW,"Done.");
}

ITEM::ITEM(char *nm)
{
  // end option - disfunctional
  IBUG(D_ITEM|D_LOW,"new ITEM[%p](\"%s\")",this,nm);
  name     = nm;
  shortcut = -1;
  callback = NULL;
  submenu  = NULL;
  next     = NULL;
  OBUG(D_ITEM|D_LOW,"Done.");
}

char ITEM::get_shortcut(void)
{
  //DBUG("ITEM(\"%s\").get_shortcut()%c%c",name,0,0);
  // Get shortcut key of option
  if (shortcut>0) return name[shortcut];
  //DBUG("ITEM(\"%s\").get_shortcut() = no shortcut%c%c",name,0,0);
  return 0;
}

void ITEM::set_shortcut(char c)
{
  // Set shortcut key of option
  int i;
  for (i=0; (name[i]!='\0') && (name[i]!=c); i++);
  if (name[i]!='\0')
    shortcut = i;
  else
    shortcut = -1;
}

char ITEM::get_free(char *s)
{
  // Return first key in option that are not in s
  int i,j;
  for (i=0; name[i]!='\0'; i++) {
    for (j=0; (s[j]!='\0') && (s[j]!=name[i]); j++);
    if (s[j]=='\0') {
      if (((name[i]>='a') && (name[i]<='z')) ||
	  ((name[i]>='A') && (name[i]<='Z')) ||
	  ((name[i]>='0') && (name[i]<='9')))
	return name[i];
    }
  }
  return 0;
}

void ITEM::set_shortcuts(char *cuts)
{
  // Set automatic shortcuts of menu
  char c = get_free(cuts);
  if (c != 0) {
    cuts[strlen(cuts)+1] = '\0';
    cuts[strlen(cuts)] = c;
    set_shortcut(c);
  }
  // Set shortcut of next
  if (next) next->set_shortcuts(cuts);
  // Set shortcut of submenu(s)
  cuts[0] = '\0';
  if (submenu) submenu->set_shortcuts(cuts);
}

ITEM *ITEM::get_next(void)
{
  // Next option in this menu
  return next;
}

int ITEM::get_shortcut_index(void)
{
  return shortcut;
}

char *ITEM::get_name(void)
{
  // Get name of this option
  return name;
}

ITEM *ITEM::get_submenu(void)
{
  // Get submenu of this option
  return submenu;
}

void ITEM::do_callback(void)
{
  // Execute callback
  if (callback) callback(name); 
}

int ITEM::num_options(void)
{
  int n = 0;
  ITEM *i = this;
  while (i) {
    ++n;
    i = i->get_next();
  }
  return n;
}

//=============================================================================

SUBMENU::SUBMENU(MENU *m,ITEM *i,WINDOW *w,COLORSET *s,PEN *p,int x,int y)
{
  IBUG(D_SUBMENU|D_MED,"new SUBMENU[%p -> %p](...) ok",w,this);
  ITEM *ti = NULL;
  int   fw = 0,
        fh = 0,
        tw = 0;

  parent_menu = m;
  items       = i;

  fh = (items->num_options() * (p->FontHeight()+4)) + 5;

  for (ti = items; ti; ti = ti->get_next()) {
    tw = p->Width(ti->get_name());
    if (tw > fw) fw = tw;
  }
  fw += p->Width(" ") + 21;
  if (Create(s->cmap()->dpy(),s->cmap(),s,0,
	     "SUBMENU","SUBMENU",w->xpos()+x,w->ypos()+y,fw,fh,
	     fw,fh,fw,fh,0,NULL,
	     0,0,p->Font(),0,NULL,WINDOWISMENU)) {
    cursor = XCreateFontCursor(display->xdpy(),XC_hand1);
    XDefineCursor(display->xdpy(),window,cursor);
    display->Add(this);
    XMapWindow(display->xdpy(),window);
    menu = new MENU(this,colorset,MENUDOWN,items);
    menu->Refresh();
    OBUG(D_SUBMENU|D_MED,"Created.");
  } else {
    OBUG(D_SUBMENU|D_MED,"Failed.");
  }
}

SUBMENU::~SUBMENU(void)
{
  if (menu) {
    menu->kill_submenu();
    delete menu;
  }
  XFreeCursor(display->xdpy(),cursor);
  //DBUG("~SUBMENU[? -> %p] ok%c%c",this,0,0);
}

void SUBMENU::Leave(int,int)
{
  if (menu && (!menu->has_submenu())) parent_menu->kill_submenu();
}

//=============================================================================

MENU::MENU(WINDOW *w,COLORSET *c,uchar m,ITEM *i) 
  : FIGLET(w,0,0,0,0)
{
  set      = c;
  items    = i;
  method   = m;
  selected = NULL;
  submenu  = NULL;
  pen      = window->dpen();
  {
    char cuts[50];
    strcpy(cuts,"");
    items->set_shortcuts(cuts);
  }
  Resize();
  window->Add(this);
  window->Motion(this);
  //  DBUG("new MENU[%p -> %p] ok%c",w,this,0);
}

MENU::~MENU(void)
{
  kill_submenu();
  window->DeMotion(this);
  window->Del(this);
  if (items && (!(method & MENUDOWN))) delete items;
  //  DBUG("~MENU[%p -> %p] ok%c",window,this,0);
}

ITEM *MENU::is_selected(int x,int y)
{
  static int   p = 0,
               w = 0,
               s = 0;
  static ITEM *i = NULL;
  if (method & MENUDOWN) {
    p = ya+4;
    s = pen->FontHeight()+4;
    for (i=items; i; i=i->get_next()) {
      if ((p<=y) && (p+s>=y)) return i;
      p += s;
    }
  } else {
    p = xa+4;
    s = pen->Width(" ");
    for (i=items; i; i=i->get_next()) {
      w = pen->Width(i->get_name());
      if ((p <= x) && (p+w>=x)) return i;
      p += w+s;
    }
  }
  return NULL;
}

void MENU::Refresh(void)
{
  pen->Foreground((*set)[ocBACKGROUND]);
  pen->Fbox(xa+2,ya+2,xb-2,yb-2);
  method |= MENUREFRESHED;
  Draw();
}

void MENU::Draw(void)
{ 
  static ITEM *oldsel     = NULL;                       // the previus selected
  char         iname[256] = "",                                   // items name
               scstr[2]   = "?";                              // items shortcut
  ulong        colsel     = (*set)[ocDARK],                   // Selected color
               colnrm     = (*set)[ocDISABLED];                 // Normal color
  int          x          = xa + 4,                             // text start x
               y          = ya + 3 + pen->FontHeight(),         // text start y
               scindex    = -1;                               // shortcut index

  if (method & MENUENTERED)
    pen->Box3d(xa,ya,xb,yb,2,(*set)[ocSHADOW],(*set)[ocHIGHLIGHT]);
  else
    pen->Box3d(xa,ya,xb,yb,2,(*set)[ocHIGHLIGHT],(*set)[ocSHADOW]);

  for (ITEM *item = items; item; item  =item->get_next()) {

    char draw = ((method & MENUREFRESHED) ||
		 (item == selected) ||
		 (item == oldsel));
    
    if (method & MENUENTERED) {               // Colors if within menu or not
      if (item == selected) {
	colsel = (*set)[ocLIGHT];
	colnrm = (*set)[ocSELECTED];
      } else {
	colsel = (*set)[ocDARK];
	colnrm = (*set)[ocFOREGROUND];
      }
    }

    scindex = item->get_shortcut_index();               // get shortcut index

    if (scindex >= 0) {                           // This item has a shortcut
      strcpy(iname,item->get_name());          // Get items name and shortcut
      scstr[0] = iname[scindex];
      iname[scindex] = '\0';
      if (scindex) {                   // one or more letters before shortcut
	if (draw) {
	  pen->Foreground(colnrm);
	  pen->Write(x,y-2,iname);
	}
	x += pen->Width(iname);
      }
      if (draw) {
	pen->Foreground(colsel);                              // Write shortcut
	pen->Write(x,y-2,scstr);
      }
      iname[scindex] = scstr[0];
	x += pen->Width(scstr);
	if ((unsigned int)(scindex+1) < strlen(iname)) {   // one or more after
	  if (draw) {
	    pen->Foreground(colnrm);
	    pen->Write(x,y-2,iname+scindex+1);
	  }
	  x += pen->Width(iname+scindex+1);
	}
	x += pen->Width(" ");
	
    } else {                              // No shortcut, do simple algorithm
      if (draw) {
	pen->Foreground(colnrm);
	pen->Write(x,y-2,item->get_name());
      }
      x += pen->Width(item->get_name()) + pen->Width(" ");
    }
    
    if (method & MENUDOWN) {                    // When the menu is a submenu
      if (draw && item->get_submenu()) {      // Draw arrow if item has submenu
	if ((item == selected) && submenu)
	  pen->Arrow(xb-14,y,xb-4,y-pen->FontHeight()+2,oARIGHT|oAFLAT|3,
		     (*set)[ocHIGHLIGHT],
		     (*set)[ocHIGHLIGHT],
		     (*set)[ocSHADOW],
		     (*set)[ocBACKGROUND]);
	else
	  pen->Arrow(xb-14,y,xb-4,y-pen->FontHeight()+2,oARIGHT|oAFLAT|3,
		     (*set)[ocSHADOW],
		     (*set)[ocSHADOW],
		     (*set)[ocHIGHLIGHT],
		     (*set)[ocBACKGROUND]);
      }
      y+=pen->FontHeight()+4;                                // One step down
      x=xa+4;
    }
  }
  oldsel = selected;
  method &= ~MENUREFRESHED;
}

char MENU::Press(uchar b,int x,int y)
{
  // Open pulldown menu
  kill_submenu();
  // Did we click a submenu ?
  if (selected && selected->get_submenu()) {
    ITEM *i = items;
    // Determine size
    if (method & MENUDOWN) {
      y = 2;
      while (i != selected) {
	y+=pen->FontHeight()+4;
	i = i->get_next();
      }
      x = xb-xa+1;
    } else {
      x = 2;
      while (i != selected) {
	x+=pen->Width(i->get_name())+pen->Width(" ");
	i = i->get_next();
      }
      y = yb-ya+1;
    }
    // Create submenu
    submenu = new SUBMENU(this,selected->get_submenu(),window,set,pen,x,y);
    Draw();
  }
  return 1;
  /*
  // Kill eventual submenus, ready optional submenu
  kill_submenu();
  if (method & MENUPRESSED) return 1;
  method |= MENUPRESSED;
  return 1;
  */
}

char MENU::Release(uchar b,int x,int y)
{
  // Execute command or kill submenu
  if (selected && !selected->get_submenu()) {
    selected->do_callback();
  }
  return 0;
  /*
  // Execute possible command
  kill_submenu();
  if (!(method & MENUPRESSED)) return 0;
  method &= ~MENUPRESSED;
  if (selected) {
    if (selected->get_submenu()) {
      ITEM *i = items;
      if (method & MENUDOWN) {
	y = 2;
	while (i != selected) {
	  y+=pen->FontHeight()+4;
	  i = i->get_next();
	}
	x = xb-xa+1;
      } else {
	x = 2;
	while (i != selected) {
	  x+=pen->Width(i->get_name())+pen->Width(" ");
	  i = i->get_next();
	}
	y = yb-ya+1;
      }
      submenu = new SUBMENU(this,selected->get_submenu(),window,set,pen,x,y);
      Draw();
    } else {
      selected->do_callback();
    }
  }
  return 1;
  */
}

char MENU::Motion(uchar,int x,int y)
{
  // Movement within text with submenu open -> kill submenu
  // Movement within arrows with submenu open -> change or keep submenu
  static ITEM *s;

  if (Inside(x,y)) {
    // if we are inside this menu, check which option we are pointing at
    s = is_selected(x,y);
    if (s != selected) {
      // We're pointing at a different option, kill any submenu, set selected
      kill_submenu();
      method |= MENUENTERED | MENUREFRESHED;
      selected = s;
      Draw();
      return 1;
    } else {
      if (!submenu && (x >= xb-14) && selected && selected->get_submenu())
    	Press(1,x,y);
    }
  }
  return 1;

  /*
  // Keep open or close menu...
  if (Inside(x,y)) {
    ITEM *s = is_selected(x,y);
    if ((!(method & MENUENTERED)) || (s != selected)) {
	kill_submenu();
	method |= MENUENTERED | MENUREFRESHED ;
	selected = s;
	Draw();
    } else {
      if (!submenu && (x >= xb-14) && selected && selected->get_submenu()) {
	Press(1,x,y);
	Release(1,x,y);
	return 0;
      }
    }
    return 1;
  } else {
    if ((method & MENUENTERED) || selected) {
      method &= ~MENUPRESSED;
      kill_submenu();
      method &= ~MENUENTERED;
      method |= MENUREFRESHED ;
      selected = NULL;
      Draw();
    }
  }
  return 0;
  */
}

void MENU::Resize(void)
{
  int w=0,fw=0,fh=0;
  if (method & MENUDOWN) {
    fh = (items->num_options() * (pen->FontHeight()+4)) + 4;
    ITEM *j = items;
    while (j) {
      w = pen->Width(j->get_name());
      if (w>fw) fw = w;
      j = j->get_next();
    }
    fw += 8+12+pen->Width(" ");
  } else {
    fw = window->Width()-1;
    fh = pen->FontHeight()+8;
  }
  FIGLET::Resize(0,0,fw,fh);
}

char MENU::has_submenu(void)
{
  return (submenu ? 1 : 0);
}

void MENU::kill_submenu(void)
{
  if (submenu) {
    delete submenu;
    submenu = NULL;
    Draw();
  }
}

void MENU::set_font(char *fnt)
{
  kill_submenu();
  pen->Font(fnt);
  Resize();
}

int MENU::pixel_width(void)
{
  return xb-xa+1;
}

int MENU::pixel_height(void)
{
  return yb-ya+1;
}

//=============================================================================
