/*
   NAME:	menu.h
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Feb 12 1997
   COMMENT:	Implementation of a menu
*/
#ifndef MENUH
#define MENUH

#include "otools.h"

class MENU;

class ITEM : public ELEMENT {
protected:
  int    shortcut;
  char  *name;
  void (*callback)(char*);
  ITEM  *submenu,*next;
public:
        ~ITEM(void);            // Delete item and options / submenus following
         ITEM(char *nm,void(*cb)(char*),ITEM *mu,ITEM *nx);             // full
         ITEM(char *nm,void(*cb)(char*),ITEM *nx);       // middle opt/callback
         ITEM(char *nm,ITEM *mu,ITEM *nx);        // middle option with submenu
         ITEM(char *nm,ITEM *nx);              // middle option - disfunctional
         ITEM(char *nm,void(*cb)(char*));           // end option with callback
         ITEM(char *nm);                          // end option - disfunctional
  char   get_shortcut(void);                      // Get shortcut key of option
  void   set_shortcut(char c);                    // Set shortcut key of option
  char   get_free(char *s);     // Return first key in option that are not in s
  void   set_shortcuts(char *cuts);          // Set automatic shortcuts of menu
  ITEM  *get_next(void);                            // Next option in this menu
  char  *get_name(void);                             // Get name of this option
  ITEM  *get_submenu(void);                       // Get submenu of this option
  void   do_callback(void);                                 // Execute callback
  int    get_shortcut_index(void);              // Get index of shortcut letter
  int    num_options(void);
};

class SUBMENU : public WINDOW {
protected:
  Cursor  cursor;
  ITEM   *items;
  MENU   *menu,
         *parent_menu;
public:
  SUBMENU(MENU *m,ITEM *i,WINDOW *w,COLORSET *s,PEN *p,int x,int y);
  ~SUBMENU(void);
  virtual void Leave(int,int);
};

/*
  The menu uses the following colors:
  Pointer outside menu:              ocDARK|ocDISABLED
  Pointer inside menu:  nonselected  ocDARK|ocFOREGROUND
                        disabled     ocDARK|ocDISABLED
			selected     ocLIGHT|ocSELECTED
 */

#define MENUDOWN      0x01
#define MENUREFRESHED 0x20
#define MENUENTERED   0x40
#define MENUPRESSED   0x80

class MENU : public FIGLET {
protected:
  uchar         method;
  ITEM         *items,*selected;
  COLORSET     *set;
  PEN          *pen;
  SUBMENU      *submenu;
public:
                MENU(WINDOW *w,COLORSET *c,uchar m,ITEM *i);
  virtual      ~MENU(void); 
  virtual void  Refresh(void);                 // Draw whole menu
  virtual void  Draw(void);                    // Draw text
  virtual char  Press(uchar,int,int);          // Notify of selection-activate
  virtual char  Motion(uchar,int x,int y);     // Select an option
  virtual char  Release(uchar,int,int);        // Activate option
  virtual ITEM *is_selected(int x,int y);      // Which item is selected in x,y
  virtual void  Resize(void);                  // Resize this menu 
  virtual char  has_submenu(void);
  virtual void  kill_submenu(void);
  virtual void  set_font(char *fnt);
  virtual int   pixel_height(void);
  virtual int   pixel_width(void);
};

#define GRAY(n)      new ITEM(n)
#define OPT(n,c)     new ITEM(n,c)
#define SUB(n,s)     new ITEM(n,(s),NULL)
#define NSUB(n,s,o)  new ITEM(n,(s),(o))
#define NOPT(n,c,o)  new ITEM(n,c,(o))

#define OPT2(a,b,c,d)\
            NOPT(a,b,OPT(c,d))
#define OPT3(a,b,c,d,e,f)\
	      NOPT(a,b,OPT2(c,d,e,f))
#define OPT4(a,b,c,d,e,f,g,h)\
	      NOPT(a,b,OPT3(c,d,e,f,g,h))
#define OPT5(a,b,c,d,e,f,g,h,i,j)\
	      NOPT(a,b,OPT4(c,d,e,f,g,h,i,j))
#define OPT6(a,b,c,d,e,f,g,h,i,j,k,l)\
	      NOPT(a,b,OPT5(c,d,e,f,g,h,i,j,k,l))
#define OPT7(a,b,c,d,e,f,g,h,i,j,k,l,m,n)\
	      NOPT(a,b,OPT6(c,d,e,f,g,h,i,j,k,l,m,n))
#define OPT8(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p)\
	      NOPT(a,b,OPT7(c,d,e,f,g,h,i,j,k,l,m,n,o,p))
#define OPT9(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r)\
	      NOPT(a,b,OPT8(c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r))
#define OPT10(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t)\
	      NOPT(a,b,OPT9(c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t))
#define OPT11(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v)\
	      NOPT(a,b,OPT10(c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v))
#define OPT12(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x)\
	      NOPT(a,b,OPT11(c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x))
#define OPT13(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z)\
	      NOPT(a,b,OPT12(c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z))
	      
#define NOPT2(a,b,c,d,z)\
	      NOPT(a,b,NOPT(c,d,z))
#define NOPT3(a,b,c,d,e,f,z)\
	      NOPT(a,b,NOPT2(c,d,e,f,z))
#define NOPT4(a,b,c,d,e,f,g,h,z)\
	      NOPT(a,b,NOPT3(c,d,e,f,g,h,z))
#define NOPT5(a,b,c,d,e,f,g,h,i,j,z)\
	      NOPT(a,b,NOPT4(c,d,e,f,g,h,i,j,z))
#define NOPT6(a,b,c,d,e,f,g,h,i,j,k,l,z)\
	      NOPT(a,b,NOPT5(c,d,e,f,g,h,i,j,k,l,z))

#define SUB2(t1,m1,t2,m2)\
	      NSUB(t1,(m1),SUB(t2,(m2)))
#define SUB3(t1,m1,t2,m2,t3,m3)\
	      NSUB(t1,(m1),SUB2(t2,(m2),t3,(m3)))
#define SUB4(t1,m1,t2,m2,t3,m3,t4,m4)\
	      NSUB(t1,(m1),SUB3(t2,(m2),t3,(m3),t4,(m4)))
#define SUB5(t1,m1,t2,m2,t3,m3,t4,m4,t5,m5)\
	      NSUB(t1,(m1),SUB4(t2,(m2),t3,(m3),t4,(m4),t5,(m5)))

#define NSUB2(t1,m1,t2,m2,o)\
	      NSUB(t1,(m1),NSUB(t2,(m2),o))
#define NSUB3(t1,m1,t2,m2,t3,m3,o)\
	      NSUB(t1,(m1),NSUB2(t2,(m2),t3,(m3),o))
#define NSUB4(t1,m1,t2,m2,t3,m3,t4,m4,o)\
	      NSUB(t1,(m1),NSUB3(t2,(m2),t3,(m3),t4,(m4),o))
#define NSUB5(t1,m1,t2,m2,t3,m3,t4,m4,t5,m5,o)\
	      NSUB(t1,(m1),NSUB4(t2,(m2),t3,(m3),t4,(m4),t5,(m5),o))

#endif /* MENUH */

