/*
   NAME:        opengl.cc
   AUTHOR:      root <root@>
   DATE:        May  8 1997
   COMMENT:     OpenGL window (RGB mode, ready for OpenGL commands)
*/

#include "opengl.h"
#include <stdlib.h>
#include <GL/glx.h>
#include <GL/gl.h>

#include "button.h"

OPENGLWINDOW::OPENGLWINDOW(DISPLAY *dpy,int _x,int _y,int w1,int h1,
			   int w2,int h2,int w3,int h3,char *csn)
  :WINDOW()
{
  // We have to do all by ourselves to create the OpenGL window we want...
  // Also, we would like to draw Otools figlets in this window too, not ??
  
  int dummy;
  int gloptions[] = { GLX_RGBA, GLX_DEPTH_SIZE,16, GLX_DOUBLEBUFFER, None };
  XVisualInfo *vi;
  GLXContext glc;
  COLORMAP *tcmap;
  COLORSET *tcset;
  
  //============================================//
  // Steps to create an OpenGL rendering window //
  //============================================//
  
  // 1. Create your display (that's done by otools)
  // 2. Make sure OpenGL's GLX extension is supported
  if (!glXQueryExtension(dpy->xdpy(),&dummy,&dummy)) {
    WARN("OpenGL windows; No X11 support found!%c%c%c",0,0,0);
    return; // This windows display is zero, that's an error for the user
  }

  // 3. Find an appropriate visual (RGB,TrueColor,depthbuffer,doublebuffer)
  vi = glXChooseVisual(dpy->xdpy(),dpy->xscr(),gloptions);
  if (vi == NULL) {
    WARN("OpenGL windows; Correct visual unavailiable!%c%c%c",0,0,0);
    return; // This windows display is zero, that's an error for the user
  }
  if ((vi->c_class) != (TrueColor)) {
    WARN("OpenGL windows; Can't use other than TrueColor!%c%c%c",0,0,0);
    return;
  }

  // 4. Create an OpenGL rendering context (no sharing, direct rendering)
  glc = glXCreateContext(dpy->xdpy(), vi, None,GL_TRUE);

  if (glc == NULL) {
    WARN("OpenGL windows; Could not create rendering context!%c%c%c",0,0,0);
    return;
  }

  // 5. Create our own colormap to use in this window (that's the trick)

  mycolormap = XCreateColormap(dpy->xdpy(),
			       RootWindow(dpy->xdpy(),vi->screen),
			       vi->visual,AllocNone);
  tcmap = new COLORMAP(dpy,mycolormap);
  dpy->Add(tcmap); // So that mycolormap will be removed together with tcmap

  // 6. Create a small colorset to use with this window
  tcset = new COLORSET(tcmap,csn); // must have at least 5 colors (!!!!)

  // 7. Create the window !!

  if (!Create(dpy,tcmap,tcset,RootWindow(dpy->xdpy(),vi->screen),
	      "OpenGL Window","OpenGL Window",_x,_y,w1,h1,w2,h2,w3,h3,
	      0,NULL,0,0,"7x13bold",vi->depth,vi->visual,0)) {
    WARN("OpenGL windows; Could not create rendering window!%c%c%c",0,0,0);
    delete tcmap;
    return;
  }
  display->Add(this);

  // 8. Bind the rendering context to the window and map it

  glXMakeCurrent(dpy->xdpy(),window,glc);
  XMapWindow(dpy->xdpy(),window);
  
  // 9. Configure the OpenGL context for rendering
  
  glEnable(GL_DEPTH_TEST);                            // enable depth buffering
  glClearColor(0.0, 0.0, 0.0, 0.0);   // frame buffer clears should be to black
  glMatrixMode(GL_PROJECTION);                   // set up projection transform
  glLoadIdentity();
  glFrustum(-1.0, 1.0, -1.0, 1.0, 1.0, 10.0);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);

  // 10. Now we're ready for some OpenGL action !!!!
  // new BUTTON(this,10,-25,-10,-5,"Hey",0,2,0,0);

  yaw = pitch = roll = 0.0;
}

void OPENGLWINDOW::Size(void)
{
  glViewport(0,0,width,height);
}

void OPENGLWINDOW::rotate(void)
{
  yaw   += 1.0;
  pitch += 2.0;
  roll  += 3.0;
}

char OPENGLWINDOW::Trigger(ELEMENT*)
{
  rotate();
  redraw();
  return 1;
}

void OPENGLWINDOW::values(float p,float t,float r)
{
  yaw = p;
  pitch = t;
  roll = r;
}

void OPENGLWINDOW::Refresh(int a,int b,int c,int d)
{
  redraw();
  WINDOW::Refresh(a,b,c,d);
}

void OPENGLWINDOW::redraw(void)
{
  glMatrixMode(GL_MODELVIEW);
  /* reset modelview matrix to the identity matrix */
  glLoadIdentity();
  /* move the object back three units */
  glTranslatef(0.0, 0.0, -3.0);
  /* rotate by X, Y, and Z angles */
  glRotatef(yaw  , 0.0, 1.0, 0.0);
  glRotatef(pitch, 1.0, 0.0, 0.0);
  glRotatef(roll , 0.0, 0.0, 1.0);

  // Refresh(0,0,width,height);

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  // front face **
  glBegin(GL_QUADS);
  glColor3f(0.0, 0.7, 0.1);       // green **
  glNormal3f(0.0,0.0,1.0);
  glVertex3f(-1.0, 1.0, 1.0);
  glVertex3f(1.0, 1.0, 1.0);
  glVertex3f(1.0, -1.0, 1.0);
  glVertex3f(-1.0, -1.0, 1.0);
  // back face **
  glColor3f(0.9, 1.0, 0.0);       // yellow **
  glNormal3f(0.0,0.0,-1.0);
  glVertex3f(-1.0, 1.0, -1.0);
  glVertex3f(1.0, 1.0, -1.0);
  glVertex3f(1.0, -1.0, -1.0);
  glVertex3f(-1.0, -1.0, -1.0);
  // top side face **
  glColor3f(0.2, 0.2, 1.0);       // blue **
  glNormal3f(0.0,1.0,0.0);
  glVertex3f(-1.0, 1.0, 1.0);
  glVertex3f(1.0, 1.0, 1.0);
  glVertex3f(1.0, 1.0, -1.0);
  glVertex3f(-1.0, 1.0, -1.0);
  // bottom side face **
  glColor3f(0.7, 0.5, 0.1);       // red **
  glNormal3f(0.0,-1.0,0.0);
  glVertex3f(-1.0, -1.0, 1.0);
  glVertex3f(1.0, -1.0, 1.0);
  glVertex3f(1.0, -1.0, -1.0);
  glVertex3f(-1.0, -1.0, -1.0);
  glEnd();
  glXSwapBuffers(xdpy(),window);
}

//=============================================================================

Material::Material(float r,float g,float b,float a,float s,float d,byte o)
{
  setColor(r,g,b);
  setSurface(a,s,d);
  setOptions(o);
}

void Material::setColor(float red,float green,float blue)
{
  color[0] = red;
  color[1] = green;
  color[2] = blue;
}
  
void Material::setSurface(float ambient,float specular,float diffuse)
{
  surface[0] = ambient;
  surface[1] = specular;
  surface[2] = diffuse;
}

void Material::setOptions(byte o)
{
  options = o;
}

void Material::activateMaterial(void)
{
  glColor3fv(color); 
  glShadeModel(options & GOURAUD ? GL_SMOOTH : GL_FLAT);
}

void Material::changeFromMaterial(Material *m)
{
  if ((color[0] != m->color[0]) || (color[1] != m->color[1]) ||
      (color[2] != m->color[2]) )
    glColor3fv(color);
  if ((options & GOURAUD) != (m->options & GOURAUD)) 
    glShadeModel(options & GOURAUD ? GL_SMOOTH : GL_FLAT);
}
    
Vertex::Vertex(float x,float y,float z)
{
  position[0] = x;
  position[1] = y;
  position[2] = z;
}

void Vertex::process(void)
{
  glVertex3fv(position);
}

NVertex::NVertex(float x,float y,float z,float e,float f,float g)
  : Vertex(x,y,z)
{
  normal[0] = e;
  normal[1] = f;
  normal[2] = g;
}

void NVertex::process(void)
{
  glVertex3fv(position);
  glNormal3fv(normal);
}
