/*!f09!M-- OTOOLS.I --*/

/*
   NAME:	otools.i
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jun 28 1996
   COMMENT:	Otools 2.0 - The inline functions of the otools classes
*/
#ifndef OTOOLSI
#define OTOOLSI

//=============================================================================
// DISPLAY
//=============================================================================

inline Window DISPLAY::xroot(void)
{
  return RootWindow(display,screen);
}

inline Display *DISPLAY::xdpy(void)
{
  return display;
}

inline int DISPLAY::xscr(void)
{
  return screen;
}

inline int DISPLAY::xfd(void)
{
  return ConnectionNumber(display);
}

inline char *DISPLAY::name(void)
{
  return title;
}

int DISPLAY::Width(void)
{
  return width;
}

int DISPLAY::Height(void)
{
  return height;
}


inline COLORMAP *DISPLAY::cmap(void)
{
  return defcmap;
}

inline void DISPLAY::Flush(void)
{
  // XSync(display,False);
  XFlush(display);
}

inline void DISPLAY::Refresh(void)
{
  WINDOW *wnd;
  long index = 1;
  while ((wnd = (WINDOW*) windows[index])) {
    wnd->Refresh(0,0,wnd->Width(),wnd->Height());
    ++index;
  }
}

inline void DISPLAY::Stop(void)
{
  running = 0;
}

inline char DISPLAY::Add(COLORMAP *cm)
{
  if (colormaps.append(cm)) return 1; else return 0;
}

inline char DISPLAY::Del(COLORMAP *cm)
{
  if (colormaps.remove(cm)) return 1; else return 0;
}

inline char DISPLAY::Add(WINDOW *wd)
{
  /// Add a window sorted by it's X11 window value
  if (windows.add(wd,(ulong)wd->xwnd())) {
    DBUG(D_DISPLAY|D_LOW,"DISPLAY[%p].Add(WINDOW[%p]) yes.",this,wd);
    return 1;
  } else {
    DBUG(D_DISPLAY|D_LOW,"DISPLAY[%p].Add(WINDOW[%p]) no.",this,wd);
    return 0;
  }
}

inline char DISPLAY::Del(WINDOW *wd)
{
  if (windows.remove(wd)) return 1; else return 0;
}

inline char DISPLAY::Exists(WINDOW *wd)
{
  if (windows.get_index(wd)) {
    DBUG(D_DISPLAY|D_GARBAGE,"DISPLAY[%p].Exists(WINDOW[%p]) yes.",this,wd);
    return 1;
  } else {
    DBUG(D_DISPLAY|D_GARBAGE,"DISPLAY[%p].Exists(WINDOW[%p]) no.",this,wd);
    return 0;
  }
}

//=============================================================================
// COLORMAP
//=============================================================================

inline char COLORMAP::Add(COLORSET *cset)
{
  if (colorsets.append(cset)) return 1; else return 0;
}

inline char COLORMAP::Del(COLORSET *cset)
{
  if (colorsets.remove(cset)) return 1; else return 0;
}

inline Colormap COLORMAP::xcmap(void)
{
  return colormap;
}

inline DISPLAY *COLORMAP::dpy(void)
{
  return display;
}

//=============================================================================
// COLORSET
//=============================================================================


inline int COLORSET::Size(void)
{
  return count;
}

inline ulong COLORSET::operator[](int ps)
{
  if (ps < 0) return set[0];                                // Index below zero
  if (ps > (count-1)) return set[count-1];                   // Index after end
  return set[ps];                                                    // Correct
}

inline COLORMAP *COLORSET::cmap(void)
{
  return colormap;
}

inline char COLORSET::Exist(int index)
{
  if (index < 0) return 1;                // Index below zero will in fact work
  if (index < count) return 1;             // Index betwwen 0 and count-1 is ok
  return 0;                                                  // Not an ok index
}

inline char *COLORSET::name(int index)
{
  if (index < 0) return NULL;
  if (index < count) return colors[index];
  return NULL;
}

//=============================================================================
// WINDOW
//=============================================================================

inline DISPLAY *WINDOW::dpy(void)
{
  return display;
}

inline Display *WINDOW::xdpy(void)
{
  return display->xdpy();
}

inline Window WINDOW::xwnd(void)
{
  return window;
}

inline ulong WINDOW::border(void)
{
  return (*colorset)[bd_idx];
}

inline ulong WINDOW::background(void)
{
  return (*colorset)[bg_idx];
}

inline ulong WINDOW::foreground(void)
{
  return (*colorset)[fg_idx];
}

inline char *WINDOW::name(void)
{
  return title;
}

inline int WINDOW::Width(void)
{
  return width;
}

inline int WINDOW::Height(void)
{
  return height;
}

inline int WINDOW::xpos(void)
{
  return posx;
}

inline int WINDOW::ypos(void)
{
  return posy;
}

inline void WINDOW::Clear(void)
{
  XClearWindow(display->xdpy(),window);
}

inline void WINDOW::Focus(FIGLET *fig)
{
  if (!fig) {
    if (focused) focused->DeFocus(fig);
    focused = fig;
  } else {
    if (fig == focused) return;
    if (fig->Focus(focused)) {
      if (focused) focused->DeFocus(fig);
      focused = fig;
    }
  }
}

inline void WINDOW::DeFocus(FIGLET *fig)
{
  if (fig == focused) {
    if (focused) focused->DeFocus(NULL);
    focused = NULL;
  }
}

inline char WINDOW::Add(PEN *p)
{
  if (pens.append(p)) return 1; else {
    WARN("WINDOW[%p].Add(PEN[%p]) failed.",this,p);
    return 0;
  }
}

inline char WINDOW::Del(PEN *p)
{
  // might be deleted twice !!
  if (pens.remove(p)) return 1; else return 0;
}

inline char WINDOW::Add(FIGLET *p)
{ 
  if (refresh.append(p)) return 1; else {
    WARN("WINDOW[%p].Add(FIGLET[%p]) failed.",this,p);
    return 0;
  }
}

inline char WINDOW::Del(FIGLET *p)
{
  // Might be deleted twice !!
  if (focused == p) focused = NULL;
  if (selected == p) selected = NULL;
  if (refresh.remove(p)) return 1; else return 0;
}

inline char WINDOW::Activate(FIGLET *f)
{
  if (active.append(f)) return 1; else return 0;
}

inline char WINDOW::DeActivate(FIGLET *f)
{
  if (selected == f) selected = NULL;
  if (active.remove(f)) return 1; else return 0;
}

inline char WINDOW::Motion(FIGLET *f)
{
  if (motion.append(f)) return 1; else return 0;
}

inline char WINDOW::DeMotion(FIGLET *f)
{
  if (selected == f) selected = NULL;
  if (motion.remove(f)) return 1; else return 0;
}

inline PEN *WINDOW::dpen(void)
{
  return pen;
}

inline COLORSET *WINDOW::cset(void)
{
  return colorset;
}

inline COLORMAP *WINDOW::cmap(void)
{
  return colormap;
}

inline void WINDOW::Resize(int w,int h)
{
  DBUG(D_WINDOW|D_GARBAGE,"WINDOW.Resize(%d,%d)",w,h);
  XResizeWindow(display->xdpy(),window,w,h);
}

inline void WINDOW::LimitSize(int miw,int mih,int maw,int mah)
{
  XSizeHints           shints;
  shints.flags          = PMinSize|PMaxSize;
  shints.min_width      = miw;
  shints.min_height     = mih;
  shints.max_width      = maw;
  shints.max_height     = mah;
  XSetWMProperties(display->xdpy(),window,NULL,NULL,NULL,0,&shints,NULL,NULL);
}

inline void WINDOW::Enter(int,int)
{
}

inline void WINDOW::Leave(int,int)
{
}

inline void WINDOW::Move(void)
{
}

inline void WINDOW::Size(void)
{
}

inline void WINDOW::Before(int,int,int,int)
{
}

inline void WINDOW::After(int,int,int,int)
{
}

//=============================================================================
// PEN
//=============================================================================

inline void PEN::Foreground(ulong color)
{
  XSetForeground(window->xdpy(),gc,color);
}

inline void PEN::Background(ulong color)
{
  XSetBackground(window->xdpy(),gc,color);
}

inline void PEN::LineAttributes(uint width,int line,int cap,int join)
{
  XSetLineAttributes(window->xdpy(),gc,width,line,cap,join);
}

inline WINDOW *PEN::wnd(void)
{
  return window;
}

inline GC PEN::xgc(void)
{
  return gc;
}

inline char PEN::Font(char *fname)
  /// Returns 1 if the gc's font was permanently changed to the new font.
  /// Returns 0 if the new font could not be loaded
{
  XFontStruct *fnt = XLoadQueryFont(window->xdpy(),fname);
  if (fnt == NULL) {
    WARN("PEN[%p].Font(\"%s\") failed.",this,fname);
    return 0;
  }
  XFreeFont(window->xdpy(),font);
  font = fnt;
  if (fontname) delete [] fontname;
  fontname = new char[strlen(fname)+1];
  strcpy(fontname,fname);
  XSetFont(window->xdpy(),gc,font->fid);                   // Set the gc's font
  return 1;
}

inline char *PEN::Font(void)
{
  // Return pointer to the pens fontname
  return fontname;
}

inline int PEN::Ascent(void)
{
  return font->ascent;
}

inline int PEN::Descent(void)
{
  return font->descent;
}

inline int PEN::FontHeight(void)
{
  return font->ascent + font->descent;
}

inline int PEN::Width(char *msg)
{
  return XTextWidth(font,msg,strlen(msg));
}

inline void PEN::Box(int xa,int ya,int xb,int yb)
{
  XDrawRectangle(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa,yb-ya);
}

inline void PEN::Fbox(int xa,int ya,int xb,int yb)
{
  XFillRectangle(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1);
}

inline void PEN::Frect(XRectangle *rect)
{
  XFillRectangles(window->xdpy(),window->xwnd(),gc,rect,1);
}

inline void PEN::Line(int xa,int ya,int xb,int yb)
{
  XDrawLine(window->xdpy(),window->xwnd(),gc,xa,ya,xb,yb);
}

inline void PEN::Clear(int xa,int ya,int xb,int yb)
{
  XClearArea(window->xdpy(),window->xwnd(),xa,ya,xb-xa,yb-ya,False);
}

inline void PEN::Write(int x,int y,char *msg)
{
  XDrawString(window->xdpy(),window->xwnd(),gc,x,y,msg,strlen(msg));
}

inline void PEN::Write3d(int x,int y,char *msg,ulong shd,ulong bgc,ulong hlt)
{
  XSetForeground(window->xdpy(),gc,hlt);
  XDrawString(window->xdpy(),window->xwnd(),gc,x-1,y-1,msg,strlen(msg));
  XDrawString(window->xdpy(),window->xwnd(),gc,x-1,y  ,msg,strlen(msg));
  XDrawString(window->xdpy(),window->xwnd(),gc,x  ,y-1,msg,strlen(msg));
  //XDrawString(window->xdpy(),window->xwnd(),gc,x+1,y-1,msg,strlen(msg));
  XSetForeground(window->xdpy(),gc,shd);
  XDrawString(window->xdpy(),window->xwnd(),gc,x+1,y+1,msg,strlen(msg));
  XDrawString(window->xdpy(),window->xwnd(),gc,x  ,y+1,msg,strlen(msg));
  //XDrawString(window->xdpy(),window->xwnd(),gc,x-1,y+1,msg,strlen(msg));
  XDrawString(window->xdpy(),window->xwnd(),gc,x+1,y  ,msg,strlen(msg));
  XSetForeground(window->xdpy(),gc,bgc);
  XDrawString(window->xdpy(),window->xwnd(),gc,x,y,msg,strlen(msg));
}

inline void PEN::Fdisjoint(int xa,int ya,int xb,int yb,int x1,int y1,int x2,
			   int y2,ulong co)
{
  int a=-1,b=-1,c=-1,d=-1;
  if (x1<xa) a = xa; else if (x1<=xb) a = x1;
  if (y1<ya) b = ya; else if (y1<=yb) b = y1;
  if (x2>xb) c = xb; else if (x2>=xa) c = x2;
  if (y2>yb) d = yb; else if (y2>=ya) d = y2;
  if (a<0 || b<0 || c<0 || d<0) {
    XSetForeground(window->xdpy(),gc,co);
    XFillRectangle(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1);
  } else {
    XSetForeground(window->xdpy(),gc,co);
    if (b-1 >= ya) Fbox(xa,ya,xb ,b-1);
    if (a-1 >= xa) Fbox(xa,b ,a-1,d  );
    if (d+1 <= yb) Fbox(xa,d+1,xb,yb);
    if (c+1 <= xb) Fbox(c+1,b,xb,d);
  }
}

inline void PEN::Justify(int *x,int *y,int xa,int ya,int xb,int yb,uchar trim,
			 char *mg)
  /// Justify text (char*) within boundaries (4 int) according to a
  /// justification field (uchar), return the Write() coordinates (2 int*)
  /// Justify() does not honor the oVWIDE and oHWIDE settings, these two
  /// are treated as oMIDDLE.
{
  int rim = trim & 0x0F; // 0->15
  if (mg==NULL) { *x = xa; *y = ya; return; }
  switch (trim & 0xC0) {
  case oRIGHT: *x = xb - rim - Width(mg) + 1; break;
  case oLEFT:  *x = xa + rim; break;
  default:     *x = (xa+xb+1-Width(mg))/2; break;
  }
  switch (trim & 0x30) {
  case oBOTTOM: *y = yb - rim - Descent(); break;
  case oTOP:    *y = ya + rim + Ascent() - 1; break;
  default:      *y = (ya+yb-Descent()+Ascent()+1)/2; break;
                  // ya0(yb-ya+1-descent-ascent)+ascent
  }
}

inline void PEN::Box3d(int xa,int ya,int xb,int yb,int num,ulong sd,ulong hl)
{
  int i;
  XSetForeground(window->xdpy(),gc,hl);
  for (i=0; i<num; i++) {
    XDrawLine(window->xdpy(),window->xwnd(),gc,xa+i,yb-i-1,xa+i,ya+i);
    XDrawLine(window->xdpy(),window->xwnd(),gc,xa+i+1,ya+i,xb-i,ya+i);
  }
  XSetForeground(window->xdpy(),gc,sd);
  for (i=0; i<num; i++) {
    XDrawLine(window->xdpy(),window->xwnd(),gc,xa+i,yb-i,xb-i,yb-i);
    XDrawLine(window->xdpy(),window->xwnd(),gc,xb-i,yb-i-1,xb-i,ya+i+1);
  }
}

inline void PEN::Fbox3d(int xa,int ya,int xb,int yb,int num,ulong sd,ulong bg,
			ulong hl)
{
  int i;
  XSetForeground(window->xdpy(),gc,hl);
  for (i=0; i<num; i++) {
    XDrawLine(window->xdpy(),window->xwnd(),gc,xa+i,yb-i-1,xa+i,ya+i);
    XDrawLine(window->xdpy(),window->xwnd(),gc,xa+i+1,ya+i,xb-i,ya+i);
  }
  XSetForeground(window->xdpy(),gc,sd);
  for (i=0; i<num; i++) {
    XDrawLine(window->xdpy(),window->xwnd(),gc,xa+i,yb-i,xb-i,yb-i);
    XDrawLine(window->xdpy(),window->xwnd(),gc,xb-i,yb-i-1,xb-i,ya+i+1);
  }
  XSetForeground(window->xdpy(),gc,bg);
  XFillRectangle(window->xdpy(),window->xwnd(),gc,xa+i,ya+i,(xb-xa)-2*i+1,
		 (yb-ya)-2*i+1);
}

inline void PEN::Diamond(int mx,int my,int r,uchar num,ulong sd,ulong hl)
{
  // r >= num !!
  while (num && r) {
    XSetForeground(window->xdpy(),gc,sd);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx,my-r,mx-r,my);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx-r,my,mx,my+r);
    XSetForeground(window->xdpy(),gc,hl);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx,my+r,mx+r,my);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx+r,my,mx,my-r);
    --r;
    --num;
  }
}

inline void PEN::Diamond(int xa,int ya,int xb,int yb,uchar num,ulong sd,
			 ulong hl)
  /// Calculates the center and radius of the diamond from the given box.
{
  int mx = (xa+xb)/2, my = (ya+yb)/2, r;
  if (((xb-xa)/2) > ((yb-ya)/2)) r = (yb-ya)/2; else r = (xb-xa)/2;
  Diamond(mx,my,r,num,sd,hl);
}

inline void PEN::Fdiamond(int mx,int my,int r,uchar num,ulong sd,ulong bg,
			  ulong hl)
{
  int s = r-num;
  XPoint p[4];
  // r >= num !!
  p[0].x = mx+1;   p[0].y = my-s-1;
  p[1].x = mx-s;   p[1].y = my;
  p[2].x = mx+1;   p[2].y = my+s+1;
  p[3].x = mx+s+1; p[3].y = my;
  XSetForeground(window->xdpy(),gc,bg);
  XFillPolygon(window->xdpy(),window->xwnd(),gc,p,4,Nonconvex,CoordModeOrigin);
  while (num && r) {
    XSetForeground(window->xdpy(),gc,sd);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx,my-r,mx-r,my);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx-r,my,mx,my+r);
    XSetForeground(window->xdpy(),gc,hl);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx,my+r,mx+r,my);
    XDrawLine(window->xdpy(),window->xwnd(),gc,mx+r,my,mx,my-r);
    --r;
    --num;
  }
}

inline void PEN::Fdiamond(int xa,int ya,int xb,int yb,uchar num,ulong sd,
			  ulong bg,ulong hl)
{
  int mx = (xa+xb)/2, my = (ya+yb)/2, r;
  if (((xb-xa)/2) > ((yb-ya)/2)) r = (yb-ya)/2; else r = (xb-xa)/2;
  Fdiamond(mx,my,r,num,sd,bg,hl);
}

inline void PEN::Clip(int xa,int ya,int xb,int yb)
{
  XRectangle rect;
  if (xa==0 && ya==0 && xb==0 && yb==0) {
    XSetClipMask(window->xdpy(),gc,0);
  } else {
    rect.x      = xa;
    rect.y      = ya;
    rect.width  = xb-xa+1;
    rect.height = yb-ya+1;
    XSetClipRectangles(window->xdpy(),gc,0,0,&rect,1,Unsorted);
  }
}

inline void PEN::Fpoly(XPoint *pt,int num,ulong fg)
{
  XSetForeground(window->xdpy(),gc,fg);
  XFillPolygon(window->xdpy(),window->xwnd(),gc,pt,num,Convex,CoordModeOrigin);
}

inline void PEN::Fcircle(int xa,int ya,int xb,int yb,ulong fg)
{
  XSetForeground(window->xdpy(),gc,fg);
  XFillArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,0,360*64);
}

inline void PEN::Round(int xa,int ya,int xb,int yb,uchar thk,ulong fg)
{
  XSetForeground(window->xdpy(),gc,fg);
  while (thk) {
    XDrawArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,0,360*64);
    xa++; ya++; xb--; yb--; thk--;
  }
  XFlush(window->xdpy());
}

inline void PEN::Fround(int xa,int ya,int xb,int yb,uchar thk,ulong sd,
			 ulong bg,ulong hl)
{
  XSetForeground(window->xdpy(),gc,hl);
  XFillArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,
	   45*64,180*64);
  XSetForeground(window->xdpy(),gc,sd);
  XFillArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,
	   225*64,180*64);
  xa+=thk; ya+=thk; xb-=thk; yb-=thk;
  XSetForeground(window->xdpy(),gc,bg);
  XFillArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,
	   0,360*64);
  window->dpy()->Flush();
  //XSetForeground(window->xdpy(),gc,bg);
  //XFillArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,0,360*64);
  //while (thk) {
  //  XSetForeground(window->xdpy(),gc,hl);
  //  XDrawArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,
  //	     230*64,-190*64);
  //  XSetForeground(window->xdpy(),gc,sd);
  //  XDrawArc(window->xdpy(),window->xwnd(),gc,xa,ya,xb-xa+1,yb-ya+1,
  //	     45*64,-185*64);
  // xa++; ya++; xb--; yb--; thk--;
  //}
}

inline void PEN::Arrow(int xa,int ya,int xb,int yb,uchar look,ulong c1,
		       ulong c2,ulong c3,ulong c4)
{
  XPoint p[3];
  int mx,my;
  float ratio;
  xa--; ya--; xb++; yb++;
  mx = (xa+xb+1)/2;
  my = (ya+yb+1)/2;
  switch (look & 0xC0) {
  case oAUP:
    ratio = (float)(xb-xa+1)/(float)(yb-ya+1);
    p[0].x = xa; p[0].y = yb;
    p[1].x = mx; p[1].y = ya;
    p[2].x = mx; p[2].y = my;
    XSetForeground(window->xdpy(),gc,c1);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[0].x = xb;
    XSetForeground(window->xdpy(),gc,c2);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[1].x = xa; p[1].y = yb;
    XSetForeground(window->xdpy(),gc,c3);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    if (look & oAFLAT) {
      p[0].x -= (int)((float)(look&0x1F)*ratio); p[0].y -= (look&0x1F)-1;
      p[1].x += (int)((float)(look&0x1F)*ratio); p[1].y -= (look&0x1F)-1;
      p[2].y  = ya + (look&0x1F)+1;
      XSetForeground(window->xdpy(),gc,c4);
      XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    }
    break;
  case oADOWN:
    ratio = (float)(xb-xa+1)/(float)(yb-ya+1);
    p[0].x = xb; p[0].y = ya;
    p[1].x = mx; p[1].y = yb;
    p[2].x = mx; p[2].y = my;
    XSetForeground(window->xdpy(),gc,c1);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[0].x = xa;
    XSetForeground(window->xdpy(),gc,c2);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[1].x = xb; p[1].y = ya;
    XSetForeground(window->xdpy(),gc,c3);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    if (look & oAFLAT) {
      p[0].x += (int)((float)(look&0x1F)*ratio); p[0].y += (look&0x1F)-1;
      p[1].x -= (int)((float)(look&0x1F)*ratio); p[1].y += (look&0x1F)-1;
      p[2].y = yb - (look&0x1F)-1;
      XSetForeground(window->xdpy(),gc,c4);
      XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    }
    break;
  case oALEFT:
    ratio = (float)(yb-ya+1)/(float)(xb-xa+1);
    p[0].x = xb; p[0].y = yb;
    p[1].x = xa; p[1].y = my;
    p[2].x = mx; p[2].y = my;
    XSetForeground(window->xdpy(),gc,c1);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[0].y = ya;
    XSetForeground(window->xdpy(),gc,c2);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[1].x = xb; p[1].y = yb;
    XSetForeground(window->xdpy(),gc,c3);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    if (look & oAFLAT) {
      p[0].x -= (look&0x1F)-1; p[0].y += (int)((float)(look&0x1F)*ratio);
      p[1].x -= (look&0x1F)-1; p[1].y -= (int)((float)(look&0x1F)*ratio);
      p[2].x = xa + (look&0x1F)+1;
      XSetForeground(window->xdpy(),gc,c4);
      XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    }
    break;
  case oARIGHT:
    ratio = (float)(yb-ya+1)/(float)(xb-xa+1);
    p[0].x = xa; p[0].y = ya;
    p[1].x = xb; p[1].y = my;
    p[2].x = mx; p[2].y = my;
    XSetForeground(window->xdpy(),gc,c1);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[0].y = yb;
    XSetForeground(window->xdpy(),gc,c2);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    p[1].x = xa; p[1].y = ya;
    XSetForeground(window->xdpy(),gc,c3);
    XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    if (look & oAFLAT) {
      p[0].x += (look&0x1F)-1; p[0].y -= (int)((float)(look&0x1F)*ratio);
      p[1].x += (look&0x1F)-1; p[1].y += (int)((float)(look&0x1F)*ratio);
      p[2].x = xb - (look&0x1F)-1;
      XSetForeground(window->xdpy(),gc,c4);
      XFillPolygon(window->xdpy(),window->xwnd(),gc,p,3,Convex,CoordModeOrigin);
    }
    break;
  }
}

//=============================================================================
// FIGLET
//=============================================================================

//=============================================================================
// FIGLET 
//=============================================================================

inline FIGLET::FIGLET(WINDOW *w)
{
  window = w;
}

inline FIGLET::FIGLET(WINDOW *w,int a,int b,int c,int d)
{
  window = w;
  FIGLET::Resize(a,b,c,d);
}

inline char FIGLET::Resize(int a,int b,int c,int d)
  /// Resizing will as default happen!
{
  xa = (a < 0) ? window->Width() +a : a;
  ya = (b < 0) ? window->Height()+b : b;
  xb = (c < 0) ? window->Width() +c : c;
  yb = (d < 0) ? window->Height()+d : d;
  if (xa>xb) { int x=xb; xb=xa; xa=x; }
  if (ya>yb) { int y=yb; yb=ya; ya=y; }
  return 1;
}

inline char FIGLET::Intersects(int a,int b,int c,int d)
{
  return ((a<=xb) && (b<=yb) && (c>=xa) && (d>=ya));
}

inline char FIGLET::Inside(int x,int y)
{
  return  ((x>=xa) && (x<=xb) && (y>=ya) && (y<=yb));
}

inline void FIGLET::Refresh(void)
  /// Force a refresh of the graphics belonging to the figlet.
  /// Default stuff is to do _nothing_.
{
}

inline char FIGLET::Press(uchar,int,int)
  /// When a button is pressed, we do what we have to, and signal if we want
  /// To be the currently selected figlet or not, in this case we don't.
{
  return 0;
}

inline char FIGLET::Motion(uchar,int,int)
  /// When the figlet is selected, and motion happens, this function will be
  /// told so, we can tell that we don't want to be selected, here we do that.
{
  return 0;
}

inline char FIGLET::Release(uchar,int,int)
  /// This is the default reply when a button is released, we don't want to
  /// be the selected figlet anymore, of course if we wanted to, we could
  /// return 1.
{
  return 0;
}

inline char FIGLET::Focus(FIGLET*)
  /// The window that owns us tells ut that it wants to give us the focus, and
  /// that FIGLET* has it right now. If we return 0 we don't want the focus,
  /// and the previous figlet will keep it, if we return 1, we will get the
  /// focus and the pending keyboard events told to Input().
{
  return 0;
}

inline void FIGLET::DeFocus(FIGLET*)
  /// Uhoh, the window doesn't want me to have focus anymore, I _must_ conform
  /// to this, no way around it. The FIGLET* tells who's grabbing the focus
  /// away from me.
{
}

inline char FIGLET::Input(char*)
  /// If I'm in focus, I might get datapackets from the keyboard filled in as
  /// char*, terminated with a '\0' character. These packages may be of any
  /// size, you may have to collect a bunch to get a full sentence.
  /// return 1 or 0, depending if you still want to keep the focus or not,
  /// in this case we don't, we return 0.
{
  return 0;
}

inline char FIGLET::ReleaseKey(char*)
{
  // Will return 1 to keep focus, so that you don't have to implement this
  // one if you're only interested in the strings entered.
  return 1;
}

inline WINDOW *FIGLET::wnd(void)
  /// Why don't I just give you the pointer to the window that owns me, you
  /// prying little function! If you want, you can define this again too,
  /// but I wouldn't mess too much with it before I understood more than the
  /// basics of otools.
{
  return window;
}

#endif /* OTOOLSI */
