/*
   NAME:	selector.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Feb  5 1997
   COMMENT:	Simple interface to select a string among strings
*/

#include "selector.h"

SELECTOR::SELECTOR(WINDOW *wnd,int a,int b,int c,int d,char *e,char f)
  : FIGLET(wnd,a,b,c,d)
{
  pen = window->dpen();
  tes = set = window->cset();
  width = (yb-ya-1)/2;
  pressed = 0;
  window->Add(this);
  window->Activate(this);
  looping = f;
  SetList(e);
}

SELECTOR::~SELECTOR(void)
{
  window->DeActivate(this);
  window->Del(this);
}

void SELECTOR::Refresh(void)
{
  // Draw border and buttons
  pen->Foreground((*set)[ocBORDER]);
  pen->Box(xa,ya,xb,yb);
  pen->Box3d(xa+1,ya+1,xb-width-1,yb-1,2,
	     (*set)[ocHIGHLIGHT],(*set)[ocSHADOW]);
  DrawArrows();
  Draw();
}

void SELECTOR::DrawArrows(void)
{
  switch (pressed) {
  case 0:
    pen->Fbox3d(xb-width,ya+1,xb-1,ya+width,1,
		(*tes)[ocSHADOW],(*tes)[ocBACKGROUND],(*tes)[ocHIGHLIGHT]);
    pen->Arrow(xb-width+2,ya+3,xb-3,ya+width-2,oAUP,
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT],
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT]);
    pen->Fbox3d(xb-width,ya+width+1,xb-1,yb-1,1,
		(*tes)[ocSHADOW],(*tes)[ocBACKGROUND],(*tes)[ocHIGHLIGHT]);
    pen->Arrow(xb-width+2,ya+width+4,xb-3,yb-3,oADOWN,
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT],
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT]);
    break;
  case 1:
    pen->Box3d(xb-width,ya+1,xb-1,ya+width,1,
	       (*tes)[ocHIGHLIGHT],(*tes)[ocSHADOW]);
    pen->Arrow(xb-width+2,ya+3,xb-3,ya+width-2,oAUP,
	       (*tes)[ocDARK],(*tes)[ocDARK],
	       (*tes)[ocDARK],(*tes)[ocDARK]);
    pen->Box3d(xb-width,ya+width+1,xb-1,yb-1,1,
	       (*tes)[ocSHADOW],(*tes)[ocHIGHLIGHT]);
    pen->Arrow(xb-width+2,ya+width+4,xb-3,yb-3,oADOWN,
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT],
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT]);
    break;
  case 2:
    pen->Box3d(xb-width,ya+1,xb-1,ya+width,1,
	       (*tes)[ocSHADOW],(*tes)[ocHIGHLIGHT]);
    pen->Arrow(xb-width+2,ya+3,xb-3,ya+width-2,oAUP,
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT],
	       (*tes)[ocLIGHT],(*tes)[ocLIGHT]);
    pen->Box3d(xb-width,ya+width+1,xb-1,yb-1,1,
	       (*tes)[ocHIGHLIGHT],(*tes)[ocSHADOW]);
    pen->Arrow(xb-width+2,ya+width+4,xb-3,yb-3,oADOWN,
	       (*tes)[ocDARK],(*tes)[ocDARK],
	       (*tes)[ocDARK],(*tes)[ocDARK]);
    break;
  }
}

void SELECTOR::Draw(void)
{
  static char current[512];
  GetString(current,number);
  // Find the string of discussion
  pen->Foreground((*set)[ocDARK]);
  pen->Fbox(xa+3,ya+3,xb-width-3,yb-3);
  pen->Foreground((*set)[ocLIGHT]);
  pen->Clip(xa+3,ya+3,xb-width-3,yb-3);
  pen->Write(xa+5,ya+pen->Ascent()+3,current);
  pen->Clip(0,0,0,0);
}

char SELECTOR::Press(uchar,int x,int y)
{
  if (x>xb-width) {
    if (y<yb-width) {
      pressed = 1;
      DrawArrows();
      return 1;
    } else {
      pressed = 2;
      DrawArrows();
      return 1;
    }
  }
  return 0;
}

char SELECTOR::Motion(uchar,int x,int y)
{
  if (!Inside(x,y)) {
    pressed = 0;
    DrawArrows();
    return 0;
  }
  return 1;
}

char SELECTOR::Release(uchar,int x,int y)
{
  if (x>xb-width) {
    if (y<yb-width) {
      if (pressed == 1) Previous();
    } else {
      if (pressed == 2) Next();
    }
  }
  pressed = 0;
  DrawArrows();
  return 0;
}

void SELECTOR::SetList(char *s)
{
  // Count number of \n's in string
  number = 0;
  list = s;
  max = 1;
  for (int i=0; list[i]!='\0'; i++) if (list[i]=='\n') ++max;
  Draw();
}

void SELECTOR::SetNumber(int n)
{
  if ((n<0) || (n>=max)) return;
  number = n;
  Draw();
}

int SELECTOR::GetNumber(void)
{
  return number;
}

void SELECTOR::GetString(char *s,int n)
{
  int i=0,j=n;
  s[0]='\0';
  if ((n<0) || (n>=max)) return;
  while ((j>0) && (list[i]!='\0')) {
    if (list[i]=='\n') --j;
    ++i;
  }
  if (list[i]=='\0')
    strcpy(s,"Illegal");
  else {
    j = 0;
    while ((list[i]!='\0') && (list[i]!='\n'))
      s[j++] = list[i++];
    s[j] = '\0';
  }
}

void SELECTOR::Next(void)
{
  ++number;
  if (number>=max) { 
    if (looping)
      number = 0;
    else
      number = max-1;
  }
  Draw();
}

void SELECTOR::Previous(void)
{
  --number;
  if (number<0) { 
    if (looping)
      number = max-1;
    else
      number = 0;
  }
  Draw();
}
