/*
   NAME:	slider.h
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 29 1996
   COMMENT:	SLIDER, a fancy single direction scroller
*/
#ifndef SLIDERH
#define SLIDERH

#include "otools.h"

/* Scroll: type-field modifiers */
#define sFLT 0x00 /* Flat, no borders (light/dark)*/
#define sSNG 0x02 /* Single 3d (in/out) */
#define sDBL 0x04 /* Double 3d (in/out) */
#define sTRP 0x06 /* Triple 3d (in/out) */
#define sLNE 0x08 /* Outlined (light+dark / dark+light) */
#define sPLV 0x0A /* split 3d vertically */
#define sPLH 0x0C /* split 3d horizontally */
#define sPLB 0x0E /* split 3d both ways */

class SLIDER:public FIGLET {
  float  hor,    /* Ranging 0 -> 1, 0,0 at left,bottom */
         ver;
  uchar  trim,   /* 2hor,2ver,4rim */
         type,   /* (from trim: 2hor,2ver),3type,1state */
         thick,  /* framewidth 0 -> */
         width,  /* 0 = only middle pixel, 1 -> */
         height, /* 0 = only middle pixel, 1 -> */
         first,  /* Wether first time draw or not */
         inuse;  /* Wether figlet is in use right now */
  int    mx,my,  /* positions from left,bottom in pixels form 0-width/height */
         tx,ty,  /* Calculated text position */
         ox,oy;  /* Old values for mx,my */
  char  *title;  /* the title */
  PEN   *pen;
  COLORSET *cset;
  void (*callback)(SLIDER*,float,float);
public:
  SLIDER(WINDOW*,                         // window
	 int,int,int,int,                 // size
	 char*,                           // name
	 uchar,uchar,                     // type,framewidth
	 uchar,uchar,                     // width,height
	 uchar,                           // trim
	 float,float,                     // hor,ver
	 void(*)(SLIDER*,float,float));   // callback
  SLIDER(WINDOW*,COLORSET*,PEN*,          // window, colorset and pen
	 int,int,int,int,                 // size
	 char*,                           // name
	 uchar,uchar,                     // type,framewidth
	 uchar,uchar,                     // width,height
	 uchar,                           // trim
	 float,float,                     // hor,ver
	 void(*)(SLIDER*,float,float));   // callback
  ~SLIDER(void);
  float Hor(void);
  float Ver(void);
  void  Values(float,float);
  void  Calculate(int,int,int*,int*,int*,int*);
  void  Catch(int,int);
  void  Refresh(void);
  void  Draw(void);
  char  Press(uchar,int,int);
  char  Motion(uchar,int,int);
  char  Release(uchar,int,int);
  char  Resize(int,int,int,int);
};

#endif /* SLIDERH */
