/*
   NAME:	switch.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 15 1996
   COMMENT:	SWITCH switch FIGLETS
*/

#include "switch.h"

SWITCH::SWITCH(WINDOW *w,int a,int b,int c,int d,char *nm,uchar tp,uchar tk,
	       uchar dp,uchar rm,uchar tr,ELEMENT *te)
  :FIGLET(w)
{
  // Set title of switch, if it fails, it will have no title
  title = NULL;
  Title(nm);
  // Switch settings
  pen  = window->dpen();
  cset = window->cset();
  type = tp;
  thick = tk;
  deep = dp;
  rim = rm;
  trim = tr;
  trigger_element = te;
  // Default colors (defined in otools.h oc?????)
  col = NULL;
  // Must recalculate switch settings at once!
  Resize(a,b,c,d);
  // Add to the window
  window->Add(this);
  window->Activate(this);
}

SWITCH::SWITCH(WINDOW *w,int a,int b,int c,int d,char *nm,uchar tp,uchar tk,
	       uchar dp,uchar rm,uchar tr,ELEMENT *te,
	       int bd,int sd,int bg,int hl,int fg,int dk,int lt)
  :FIGLET(w)
{
  // Set title of switch, if it fails, it will have no title
  title = NULL;
  Title(nm);
  // Switch settings
  pen  = window->dpen();
  cset = window->cset();
  type = tp;
  thick = tk;
  deep = dp;
  rim = rm;
  trim = tr;
  trigger_element = te;
  // Default colors (defined in otools.h oc?????)
  if ((col = new int[7])) {
    col[0] = bd;
    col[1] = sd;
    col[2] = bg;
    col[3] = hl;
    col[4] = fg;
    col[5] = dk;
    col[6] = lt;
  }
  // Must recalculate switch settings at once!
  Resize(a,b,c,d);
  // Add to the window
  window->Add(this);
  window->Activate(this);
}

SWITCH::~SWITCH(void)
{
  // We're not active anymore
  window->DeActivate(this);
  window->Del(this);
  // Free titlememory
  if (title) delete [] title;
}

const char *SWITCH::Title(char *tt)
  /// If tt is NULL, the titles string is returned
  /// Else tt is attempted copied into the title, NULL returned if it failed
{
  if (!tt) return (const char*)title;
  if (title) {
    char *t = new char[strlen(tt)+1];
    if (!t) {
      WARN("SWITCH[%p].Title(\"%s\") could not allocate titlememory",
	   this,tt);
      return NULL;
    }
    strcpy(t,tt);
    delete [] title;
    title = t;
  } else {
    title = new char[strlen(tt)+1];
    if (!title) return NULL;
    strcpy(title,tt);    
  }
  return title;
}

char SWITCH::Resize(int a,int b,int c,int d)
{
  FIGLET::Resize(a,b,c,d);
  a = xa; b = ya; c = xb; d = yb;
  // Inner frame size
  if (type & swFRAME) { ++a; ++b; --c; --d; }
  if (type & swBORDER) { a+=2; b+=2; c-=2; d-=2; }
  // Text placement
  switch (type & 0xC0) {
  case swLEFT:  a += deep+1; break;
  case swABOVE: b += deep+1; break;
  case swRIGHT: c -= deep+1; break;
  case swBELOW: d -= deep+1; break;
  }
  if (title) pen->Justify(&tx,&ty,a,b,c,d,trim,title);
  // calculate switch size
  switch (type & 0xC0) {
  case swLEFT:  c = a-1; a -= deep+1; break;
  case swABOVE: d = c-1; c -= deep+1; break;
  case swRIGHT: a = c+1; c += deep+1; break;
  case swBELOW: b = d+1; d += deep+1; break;
  }
  a+=rim; b+=rim; c-=rim; d-=rim;
  // The switch is quadratic
  if ((c-a) < (d-b)) {
    int n = (d-b) - (c-a);
    b += n/2;
    d -= (n+1)/2;
  } else
  if ((d-b) < (c-a)) {
    int n = (c-a) - (d-b);
    a += n/2;
    c -= (n+1)/2;
  }
  // Must have a center pixel
  if ((d-b) % 2) {
    c++; d++;
  }
  sxa = a; sya = b; sxb = c; syb = d;
  return 1;
}

void SWITCH::Refresh(void)
{
  int a = xa,b = ya,c = xb,d = yb;
  if (type & swFRAME) {
    pen->Foreground((*cset)[col ? col[0] : ocBORDER]);
    pen->Box(a,b,c,d);
    ++a; ++b; --c; --d;
  }
  if (type & swBORDER) {
    if (type & swSUNKEN)
      pen->Fbox3d(a,b,c,d,2,
		  (*cset)[col ? col[3] : ocHIGHLIGHT],
		  (*cset)[col ? col[2] : ocBACKGROUND],
		  (*cset)[col ? col[1] : ocSHADOW]);
    else
      pen->Fbox3d(a,b,c,d,2,
		  (*cset)[col ? col[1] : ocSHADOW],
		  (*cset)[col ? col[2] : ocBACKGROUND],
		  (*cset)[col ? col[3] : ocHIGHLIGHT]);
  } else {
    pen->Foreground((*cset)[col ? col[2] : ocBACKGROUND]);
    pen->Fbox(a,b,c,d);
  }
  if (title) {
    pen->Foreground((*cset)[col ? col[4] : ocFOREGROUND]);
    pen->Write(tx,ty,title);
  }
  Draw();
}

void SWITCH::Draw(void)
{
  switch (type & 0x30) {
  case swRECT:
    if (type & swON)
      pen->Fbox3d(sxa,sya,sxb,syb,thick,
		  (*cset)[col ? col[3] : ocHIGHLIGHT],
		  (*cset)[col ? col[7] : ocLIGHT],
		  (*cset)[col ? col[1] : ocSHADOW]);
    else
      pen->Fbox3d(sxa,sya,sxb,syb,thick,
		  (*cset)[col ? col[1] : ocSHADOW],
		  (*cset)[col ? col[6] : ocDARK],
		  (*cset)[col ? col[3] : ocHIGHLIGHT]);
    break;
  case swDIAMOND:
    if (type & swON)
      pen->Fdiamond(sxa,sya,sxb,syb,thick,
		    (*cset)[col ? col[1] : ocSHADOW],
		    (*cset)[col ? col[7] : ocLIGHT],
		    (*cset)[col ? col[3] : ocHIGHLIGHT]);
    else
      pen->Fdiamond(sxa,sya,sxb,syb,thick,
		    (*cset)[col ? col[3] : ocHIGHLIGHT],
		    (*cset)[col ? col[6] : ocDARK],
		    (*cset)[col ? col[1] : ocSHADOW]);
    break;
  case swROUND:
    if (type & swON)
      pen->Fround(sxa,sya,sxb,syb,thick,
		  (*cset)[col ? col[3] : ocHIGHLIGHT],
		  (*cset)[col ? col[7] : ocLIGHT],
		  (*cset)[col ? col[1] : ocSHADOW]);
    else
      pen->Fround(sxa,sya,sxb,syb,thick,
		  (*cset)[col ? col[1] : ocSHADOW],
		  (*cset)[col ? col[6] : ocDARK],
		  (*cset)[col ? col[3] : ocHIGHLIGHT]);
    break;
  case swPIN:
    if (type & swON) {
      int w = (sxb-sxa+1)/4,h = (syb-sya+1)/4;
      static XPoint p[3];
      pen->Fround(sxa,sya,sxa+(sxb-sxa+1)/2,sya+(syb-sya+1)/2,thick,
		  (*cset)[col ? col[3] : ocHIGHLIGHT],
		  (*cset)[col ? col[2] : ocBACKGROUND],
		  (*cset)[col ? col[1] : ocSHADOW]);
      p[0].x = sxa+w;     p[0].y = sya+h;
      p[1].x = sxb-w+w/2; p[1].y = syb-h-h/2;
      p[2].x = sxb-w-w/2; p[2].y = syb-h+h/2;
      pen->Fpoly(p,3,(*cset)[col ? col[6] : ocDARK]);
      pen->Fcircle(sxa+(sxb-sxa+1)/2,sya+(syb-sya+1)/2,sxb,syb,
		   (*cset)[col ? col[7] : ocLIGHT]);
    } else {
      pen->Foreground((*cset)[col ? col[2] : ocBACKGROUND]);
      pen->Fbox(sxa,sya,sxb,syb);
      pen->Fround(sxa,sya,sxa+(sxb-sxa+1)/2,sya+(syb-sya+1)/2,thick,
		  (*cset)[col ? col[1] : ocSHADOW],
		  (*cset)[col ? col[2] : ocBACKGROUND],
		  (*cset)[col ? col[3] : ocHIGHLIGHT]);
    }
    break;
  }
}

char SWITCH::Press(uchar,int,int)
{
  type ^= swON;
  Draw();
  //DBUG("SWITCH.Press = 1%c%c%c",0,0,0);
  return 1;
}

char SWITCH::Motion(uchar,int x,int y)
{
  if (!Inside(x,y)) {
    type ^= swON;
    Draw();
    //DBUG("SWITCH.Motion = 0%c%c%c",0,0,0);
    return 0;
  }
  //DBUG("SWITCH.Motion = 1%c%c%c",0,0,0);
  return 1;
}

char SWITCH::Release(uchar,int,int)
{
  if (trigger_element) 
    trigger_element->Trigger(this);
  else
    Trigger(this);
  //DBUG("SWITCH.Release = 0%c%c%c",0,0,0);
  return 0;
}

char SWITCH::On(void)
{
  return (type & swON);
}

void SWITCH::State(uchar st)
{
  if (st & swON)
    type |= swON;
  else
    type &= ~swON;
  Draw();
}
