/*
   NAME:	switch.h
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 15 1996
   COMMENT:	SWITCH switch FIGLETS
*/
#ifndef SWITCHH
#define SWITCHH

#include "otools.h"

#define swBELOW   0xC0 // position of switch to text (2bit)
#define swABOVE   0x80
#define swRIGHT   0x40
#define swLEFT    0x00
#define swPIN     0x30 // look of switch (2bit)
#define swROUND   0x20
#define swDIAMOND 0x10
#define swRECT    0x00
#define swON      0x08 // Switch is ON (1bit)
#define swFRAME   0x04 // frame/3dframe/if frame is sunken (3bit)
#define swBORDER  0x02
#define swSUNKEN  0x01

/*
  type:
  swLEFT,swRIGHT,swABOVE,swBELOW     position of switch to text
  swON                               switch is ON
  swRECT,swDIAMOND,swROUND,swPIN     look of switch
  swFRAME                            frame around switch
  swBORDER                           3d frame around switch
  swSUNKEN                           if frame is sunken

  +-------------------------+ Switch with swLEFT set.
  |     | rim: +----------+ |
  |   +--+   : |          | |
  |   |sw|   : |          | |
  |<->|it|<->: |  title   | | The switch will be quadratic
  |rim|ch|rim: |          | |
  |   +--+   : |(trim)    | |
  |     | rim: +----------+ |
  +-------------------------+
  <------------>
       deep

*/
 
class SWITCH:public FIGLET {
protected:
  PEN      *pen;
  COLORSET *cset;
  uchar     type,thick,deep,rim,trim;
  int       tx,ty,sxa,sya,sxb,syb,*col;
  ELEMENT  *trigger_element;
  char     *title;
public:
  SWITCH(WINDOW*,      // The window of the switch
	 int,int,      // Upper left corner of switch
	 int,int,      // Lower right corner of switch
	 char*,        // The title of the switch
	 uchar,        // type: swLEFT/RIGHT/ABOVE/BELOW|BORDER|FRAME|SUNKEN|ON
	 uchar,        // thick: 0-255 (thickness of switch)
	 uchar,        // deep: 0-255 (space allowed for switch)
	 uchar,        // rim: 0-255 (space around switch)
	 uchar,        // trim: oLEFT/RIGHT/MIDDLE/HWIDE/TOP/BOTTOM/VWIDE|0-15
	 ELEMENT*);    // element to Trigger() (NULL = this)
  SWITCH(WINDOW*,int,int,int,int,char*,uchar,uchar,uchar,uchar,uchar,ELEMENT*,
	 int,int,int,int,int,int,int); // As above: bd,sd,bg,hl,fg,dk,lt
  virtual ~SWITCH(void);
  virtual const char *Title(char*); // parameter = NULL -> return title
  virtual char Resize(int,int,int,int);
  virtual void Refresh(void);
  virtual void Draw(void);
  virtual char Press(uchar,int,int);
  virtual char Motion(uchar,int,int);
  virtual char Release(uchar,int,int);
  virtual char On(void);
  virtual void State(uchar);
};

#endif /* SWITCHH */
