/*
   NAME:        text.cc
   AUTHOR:      root <root@>
   DATE:        Dec 17 1996
   COMMENT:     TEXT figlet, for simple formatted text
*/

#include <stdlib.h>
#include "text.h"

TEXT::TEXT(WINDOW *wn,COLORSET *cs,PEN *pn,
	   int a,int b,int c,int d,
	   char *txt,uchar lk)
  : FIGLET(wn,a,b,c,d)
{
  //IBUG("new TEXT[%p](window[%p],\"%s\")",this,wn,txt);
  set = cs ? cs : window->cset();
  pen = pn ? pn : window->dpen();
  font_height = pen->FontHeight();
  pixmapcreated = 0;
  horisontal_scroll = vertical_scroll = 0;
  widths = NULL;
  text = NULL;
  text_count = text_size = 0;
  look = lk;
  max_width = max_height = 0;
  tab_spaces = 8;
  Resize(xa,ya,xb,yb);
  Format(txt); // Use this text
  window->Add(this);
  //OBUG("new TEXT()%c%c%c",0,0,0);
}
  
TEXT::~TEXT(void)
{
  //IBUG("~TEXT[%p]()%c%c",this,0,0);
  if (pixmapcreated) {
    XFreePixmap(window->xdpy(),pixbuf);
    pixmapcreated = 0;
  }
  window->Del(this);
  Empty();
  Format(NULL);
  //OBUG("~TEXT()%c%c%c",0,0,0);
}

char TEXT::Resize(int a,int b,int c,int d)
{
  //IBUG("TEXT[%p].Resize(%d,%d)",this,c-a+1,d-b+1);
  Window root;
  int xp,yp;
  uint wd,ht,dw,dp;
  FIGLET::Resize(a,b,c,d);
  if (pixmapcreated) {
    XFreePixmap(window->xdpy(),pixbuf);
    pixmapcreated = 0;
  }
  XGetGeometry(window->xdpy(),window->xwnd(),&root,&xp,&yp,&wd,&ht,&dw,&dp);
  pixmapcreated = 1;
  pixbuf = XCreatePixmap(window->xdpy(),window->xwnd(),xb-xa+1,yb-ya+1,dp);
  //OBUG("TEXT.Resize()%c%c%c",0,0,0);
  return 1;
}

void TEXT::Empty(void)
{
  //IBUG("TEXT[%p].Empty()%c%c",this,0,0);
  if (!text) {
    //OBUG("TEXT.Empty() no text to empty.%c%c%c",0,0,0);
    return;
  }
  for (ulong i=0; i<text_count; i++) {
    delete [] text[i];
  }
  delete [] text;
  text = NULL;
  text_size = 0;
  text_count = 0;
  delete [] widths;
  widths = 0;
  max_height = 0;
  if (!(look & txtBOX))
    max_width = 0;
  //OBUG("TEXT.Empty() emptied%c%c%c",0,0,0);
}

char TEXT::Append(uchar *fm,uchar *fg,uchar *bg,uchar *cr,char *txt)
{
  //DBUG("TEXT[%p].Append(\"%s\")%c",this,txt,0);
  // Append a single line to the text-array
  // The initial format is passed in [fm,fg,bg,cr] These parameters
  // reflect the format after this line of text.
  if (!txt) return 1;
  // Create array if it doesn't exist 
  if (!text) {
    text_size = 50;
    text = new char*[text_size];
    widths = new ulong[text_size];
    text_count = 0;
    if (!text || !widths) {
      WARN("TEXT[%p].Append(\"%s\"): Cannot create array to append a line!",
	   this,txt);
      return 0;
    }
  }
  // Enlarge array if it is too small
  if (text_count >= text_size) {
    text_size += text_size/2; // 50, 75, 137 , 205 etc 
    ulong *u = new ulong[text_size];
    char **t = new char*[text_size];
    if (!t) {
      WARN("TEXT[%p].Append(\"%s\"): Cannot resize array to append a line!",
	   this,txt);
      return 0;
    }
    for (ulong i=0; i<text_count; i++) {
      t[i] = text[i]; // Mark that the pointers are copied.
      u[i] = widths[i];
    }
    delete [] text;
    delete [] widths;
    text = t; // The lines themselves are kept, only the index is changed
    widths = u;
  }
  // Add textline with extra bytes
  text[text_count] = new char[strlen(txt)+5];
  strcpy(text[text_count]+4,txt);
  (text[text_count])[0] = *fm;
  (text[text_count])[1] = *fg;
  (text[text_count])[2] = *bg;
  (text[text_count])[3] = *cr;

  ++text_count;

  // Calculate format at end of this line (!) together with width
  widths[text_count-1] = Width(fm,fg,bg,cr,text_count-1);

  if (widths[text_count-1] > max_width) max_width = widths[text_count-1];

  // Correct for justification commands
  if ((*fm & 0xC0) != (text[text_count-1][0] & 0xC0)) {
    text[text_count-1][0] &= 0x3F;
    text[text_count-1][0] |= (*fm & 0xC0);
  }

  max_height += font_height;

  return 1;
}

char TEXT::Token(uchar *fm,uchar *fg,uchar *bg,uchar *cr,ulong line,ulong *i,
		 char *token)
{
  //DBUG("TEXT[%p].Token(%ld,%ld)",this,line,*i);
  // Returns the format and string-token starting at position i in line 'line'
  // Return value 0 means there's no more tokens on this line, format correct
  // Will convert TAB characters to SPACE

  // No such line
  if (line >= text_count) return 0;

  // After end of line
  if (*i >= ((ulong)strlen(text[line]+4)+4)) return 0;

  // Check for 'erroneous' i
  if (*i < 4) {
    *fm = text[line][0];
    *fg = text[line][1];
    *bg = text[line][2];
    *cr = text[line][3];
    *i = 4;
  }

  // While there is format codes here

  while ((text[line][*i] == '!') && (text[line][*i+1] !='!')) {

    (*i)++;
    switch (text[line][*i]) {

    // Turn off BOLD / UNDERLINE 
    case 'N':
      *fm &= ~(txtULINE|txtBOLD);
      (*i)++;
      break;

    case 'B':
      *fm ^= txtBOLD;
      (*i)++;
      break;

    case 'U':
      *fm ^= txtULINE;
      (*i)++;
      break;

    case 'L':
      *fm &= 0x3F;
      *fm |= oLEFT;
      (*i)++;
      break;

    case 'R':
      *fm &= 0x3F;
      *fm |= oRIGHT;
      (*i)++;
      break;

    case 'W':
      *fm &= 0x3F;
      *fm |= oHWIDE;
      (*i)++;
      break;

    case 'M':
      *fm &= 0x3F;
      (*i)++;
      break;

    case 'f':
      {
	char hex[3];
	(*i)++; hex[0] = text[line][*i];
	(*i)++; hex[1] = text[line][*i];
	(*i)++; hex[2] = '\0';
	*fg = (uchar) set->cmap()->Value(hex);
	// WARN("Color %s=%ld%c",hex,*fg,0);
      }
      break;

    case 'b':
      {
	char hex[3];
	(*i)++; hex[0] = text[line][*i];
	(*i)++; hex[1] = text[line][*i];
	(*i)++; hex[2] = '\0';
	*bg = (uchar) set->cmap()->Value(hex);
      }
      break;

    case 'c':
      {
	char hex[3];
	(*i)++; hex[0] = text[line][*i];
	(*i)++; hex[1] = text[line][*i];
	(*i)++; hex[2] = '\0';
	*cr = (uchar) set->cmap()->Value(hex);
      }
      break;

    default:
      (*i)++;
      break;

    }

    // MUST DO
  }

  // No tokens after format spesifications, end this
  if (text[line][*i] == '\0') {
    token[0]='\0';
    return 0;
  }

  // Collect the string with this format
  ulong j = 0;
  // Collect a token from the line (with no formats)
  while (text[line][*i] != '\0') {
    
    // oHWIDE tokenizing (spaces are token dividers)
    if (((*fm & 0xC0) == oHWIDE) && (j > 0)) { // Block justification
      if (token[j-1] == ' ') {
	if (text[line][*i] != ' ') { // Another token here !!
	  token[j]='\0';
	  return 1;
	}
      } else {
	if (text[line][*i] == ' ') { // Another token here !!
	  token[j]='\0';
	  return 1;
	}
      }
    }

    // WARN("koko %c,%c->%c",text[line][*i],text[line][*i+1],token[j]);
    // Found possible format
    if (text[line][*i] == '!') {
      switch (text[line][*i+1]) {
      // End? Treat is as an exclamation...
      case '\0':
	break;
      // Just an exclamation
      case '!':
	(*i)++;
        break;
      // It's a format, exit now!
      default:
	token[j] = '\0';
	return 1;
	break;
      }
    }
    
    // Collect this part of the token
    token[j++] = text[line][(*i)++];
    // Tab token
    if (token[j-1]=='\t') {
      --j;
      for (int kk=0; kk<tab_spaces; kk++) {
	token[j++] = ' ';
      }
    }
  } 

  // End of line, this is a token!
  token[j] = '\0';
  return 1;
}

ulong TEXT::Width(uchar *fm,uchar *fg,uchar *bg,uchar *cr,ulong line)
{
  ulong width = 0;
  char token[512];
  ulong index = 0;
  // Get endformat and width
  while (Token(fm,fg,bg,cr,line,&index,token)) {
    if (*fm & txtBOLD) ++width;
    width += pen->Width(token);
  }
  return width;
}

void TEXT::Format(char *txt)
{
  uchar format,fgcolor,bgcolor,crcolor;
  // Formats the text-input so that it can be used with this figlet.
  // This will act as AppendFormat if there is text here from before
  char line[512]; 
  if (text_count == 0) {
    // Default format...
    format = oLEFT|2;
    fgcolor = ocFOREGROUND;
    bgcolor = ocBACKGROUND;
    crcolor = ocLIGHT;
  } else {
    // Get format from last string
    // Calculate format at end of line (Not done yet)
    Width(&format,&fgcolor,&bgcolor,&crcolor,text_count-1);
  }
  // Do nothing if there's no text to format
  if (!txt) return;
  
  // For whole textbuffer
  ulong i = 0;
  while (txt[i]!='\0') {

    // Next line
    ulong j = 0;
    while ((txt[i]!='\0') && (txt[i]!='\n') && (txt[i]!='\r'))
      line[j++] = txt[i++];
    line[j] = '\0';

    // Append line with initial format, get next lines format
    // max_height and max_width is updated.
    if (line[0] != '\0')
      Append(&format,&fgcolor,&bgcolor,&crcolor,line);

    // Step over \n \r
    while ((txt[i]=='\r') || (txt[i]=='\n')) i++;
  }
}

void TEXT::Scroll(float h,float v)
{
  char fresh = 0;
  ulong hs = horisontal_scroll,
        vs = vertical_scroll;
  if (h >= 0.0) {
    hs = (ulong)(h*max_width);
    fresh = 1;
    hs /= pen->Width("o");
    hs *= pen->Width("o");
  }
  if (v >= 0.0) {
    vs   = (ulong)(v*max_height);
    fresh = 1;
    vs /= font_height;
    vs *= font_height;
  }
  if (fresh) Draw(hs,vs,0);
}

void TEXT::Refresh(void)
{
  //DBUG("TEXT[%p].Refresh()%c%c",this,0,0);
  Freshen(xa,ya,xb,yb);
}

void TEXT::Freshen(int ixa,int iya,int ixb,int iyb)
{
  //DBUG("TEXT[%p].Freshen(ixa=%d,iya=%d)",this,ixa,iya);
  pen->Foreground((*set)[text_count ? text[0][2] : ocBACKGROUND]);
  pen->Fbox(ixa,iya,ixb,iyb);

  // Which line to start with, how many lines to draw
  ulong start  = vertical_scroll / font_height,
        number = (yb-ya+1+pen->Ascent()) / font_height;
  if ((yb-ya+1+pen->Ascent()) % font_height) ++number;
  int yoff = ya + pen->Ascent() - (vertical_scroll % font_height);

  pen->Foreground((*set)[text_count ? text[0][1] : ocFOREGROUND]);
  pen->Clip(ixa,iya,ixb,iyb);

  for (ulong i=0; (start+i < text_count) && (i < number); ++i) {

    int   x, y = yoff + (i * font_height);
    uchar fm, fg, bg, cr;
    char  token[512];
    ulong index = 0;
    long  width = 0;

    if (y>(iyb+font_height)) break;
    
    if (y>=iya) {
      switch (text[start+i][0] & 0xC0) {
      case oHWIDE:
      case oLEFT:
	x = xa - horisontal_scroll;
	break;
      case oRIGHT:
	if (look & txtBOX)
	  x = xb - horisontal_scroll - widths[start+i] + 1;
	else
	  x = xa - horisontal_scroll + max_width - widths[start+i];
	break;
      default: // case oMIDDLE:
	if (look & txtBOX) 
	  x = (xa + xb + 1 - widths[start+i]) / 2 - horisontal_scroll;
	else
	x = xa + (max_width - widths[start+i]) / 2 - horisontal_scroll;
	break;
      }

      if ((text[start+i][0] & 0xC0) == oHWIDE) {
	long gaps = 0,gap,each,extra;
	// count gaps
	while (Token(&fm,&fg,&bg,&cr,start+i,&index,token)) {
	  if (token[0] == ' ') {
	    ++gaps;
	    while (Token(&fm,&fg,&bg,&cr,start+i,&index,token) &&
		   (token[0]==' '));
	  }
	}
	
	if (look & txtBOX)
	  gap = (xb-xa+1) - widths[start+i];
	else
	  gap = max_width - widths[start+i];
	
	if (gaps == 0) {
	  each = extra = 0;
	} else if (gap <= 0) {
	  gap *= -1;
	  each = -(gap / gaps);
	  extra = -(gap % gaps);
	} else {
	  each  = gap / gaps;
	  extra = gap % gaps;
	}
	
	char space = 0;
	int xadd = 0;
	index = 0;
	
	while ((x<=ixb) && Token(&fm,&fg,&bg,&cr,start+i,&index,token)) {
	  width = pen->Width(token);
	  if (token[0]==' ') {
	    if (!space) {
	      space = 1; xadd = each;
	      if (extra>0)
		{ ++xadd; --extra; }
	      else
		{ --xadd; ++extra; }
	    }
	  } else space=0;
	  
	  if (bg != ocBACKGROUND) {
	    pen->Foreground((*set)[bg]);
	    pen->Fbox(x,y-pen->Ascent()+1,x+width+xadd,y+pen->Descent());
	  }
	  if (((x+width+xadd)>=ixa) && (token[0] != ' ')) {
	    pen->Foreground((*set)[fg]);
	    pen->Write(x+xadd,y,token);
	  }
	  if (fm & txtBOLD) {
	    if (((x+width+xadd)>=ixa-1) && (token[0] != ' '))
	      pen->Write(x+1+xadd,y,token);
	    ++width;
	  }
	  if (fm & txtULINE)
	    pen->Line(x+xadd,y+1,x+width,y+1);
	  x += width+xadd;
	  xadd = 0;
	}
	
      } else {
	
	while ((x<=ixb) && Token(&fm,&fg,&bg,&cr,start+i,&index,token)) {
	  width = pen->Width(token);
	  if (bg != ocBACKGROUND) {
	    pen->Foreground((*set)[bg]);
	    pen->Fbox(x,y-pen->Ascent()+1,x+width,y+pen->Descent());
	  }
	  pen->Foreground((*set)[fg]);
	  pen->Write(x,y,token);
	  if (fm & txtBOLD) {
	    pen->Write(x+1,y,token);
	    ++width;
	  }
	  if (fm & txtULINE)
	    pen->Line(x,y+1,x+width,y+1);
	  x += width;
	}
      }
    }
  }
  pen->Clip(0,0,0,0);
}

void TEXT::Draw(ulong new_x,ulong new_y,char redraw)
{
  //DBUG("TEXT[%p].Draw(x=%ld,y=%ld)",this,new_x,new_y);
  if (redraw) {
    // AREA 1 - redraw, set rectangle to whole window
    horisontal_scroll = new_x;
    vertical_scroll = new_y;
    Freshen(xa,ya,xb,yb);
  } else {
    // AREA 2 - move buffer according to xypos, set rectangle to touched area
    // find area(s) to refresh... (1 or 3 areas)
    long wide = horisontal_scroll - new_x,
         high = vertical_scroll - new_y;
    horisontal_scroll = new_x;
    vertical_scroll = new_y;
    if ((wide == 0) && (high == 0)) return; // no movement...
    if ( (abs(wide) > (xb-xa+1)) || (abs(high) > (yb-ya+1)) ) {
      Freshen(xa,ya,xb,yb);
      return;
    }
    if (wide == 0) { // only vertical movement...
      if (high<0) { // scrolling down
	XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		  xa,ya-high,xb-xa+1,yb-ya+1+high,0,0);
	XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		  0,0,xb-xa+1,yb-ya+1+high,xa,ya);
	Freshen(xa,yb+high,xb,yb);
      } else { // scrolling up
	XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		  xa,ya,xb-xa+1,yb-ya+1-high,0,0);
	XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		  0,0,xb-xa+1,yb-ya+1-high,xa,ya+high);
	Freshen(xa,ya,xb,ya+high);
      }
    } else if (high == 0) { // horisontal movement
      if (wide<0) { // Scrolling to the left
	XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		  xa-wide,ya,xb-xa+1+wide,yb-ya+1,0,0);
	XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		  0,0,xb-xa+1+wide,yb-ya+1,xa,ya);
	Freshen(xb+wide,ya,xb,yb);
      } else { // Scrolling to the right
	XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		  xa,ya,xb-xa+1-wide,yb-ya+1,0,0);
	XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		  0,0,xb-xa+1-wide,yb-ya+1,xa+wide,ya);
	Freshen(xa,ya,xa+wide,yb);
      }
    } else { // scrolling both ways
      if (wide<0) { // scrolling to the left
	if (high<0) { // scrolling down
	  XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		    xa-wide,ya-high,xb-xa+1+wide,yb-ya+1+high,0,0);
	  XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		    0,0,xb-xa+1+wide,yb-ya+1+high,xa,ya);
	  Freshen(xa,yb+high,xb,yb);
	  Freshen(xb+wide,ya,xb,yb+high-1);
	} else { // scrolling up
	  XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		    xa-wide,ya,xb-xa+1+wide,yb-ya+1-high,0,0);
	  XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		    0,0,xb-xa+1+wide,yb-ya+1-high,xa,ya+high);
	  Freshen(xa,ya,xb,ya+high);
	  Freshen(xb+wide,ya+high+1,xb,yb);
	}
      } else { // scrolling to the right
	if (high<0) { // scrolling down
	  XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		    xa,ya-high,xb-xa+1-wide,yb-ya+1+high,0,0);
	  XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		    0,0,xb-xa+1-wide,yb-ya+1+high,xa+wide,ya);
	  Freshen(xa,yb+high,xb,yb);
	  Freshen(xa,ya,xa+wide,yb+high-1);
	} else { // scrolling up
	  XCopyArea(window->xdpy(),window->xwnd(),pixbuf,pen->xgc(),
		    xa,ya,xb-xa+1-wide,yb-ya+1-high,0,0);
	  XCopyArea(window->xdpy(),pixbuf,window->xwnd(),pen->xgc(),
		    0,0,xb-xa+1-wide,yb-ya+1-high,xa+wide,ya+high);
	  Freshen(xa,ya,xb,ya+high);
	  Freshen(xa,ya+high+1,xa+wide,yb);
	}
      }
    }
  }
}

void TEXT::Show(void)
{
  for (ulong i=0; i<text_count; i++) 
    printf("%s\n",text[i]+4);
}

char TEXT::Load(uchar *fm,uchar *fg,uchar *bg,uchar *cr,char *filename)
{
  char s[1024];
  FILE *fin = fopen(filename,"rb");
  if (!fin) return 0;
  while (!feof(fin)) {
    if (fgets(s,1023,fin)) {
      s[strlen(s)-1] = '\0';
      Append(fm,fg,bg,cr,s);
    }
  }
  fclose(fin);
  return 1;
}

void TEXT::TextSize(ulong *w,ulong *h)
{
  *w = max_width;
  *h = max_height;
}




