/*
   NAME:        text.h
   AUTHOR:      root <root@>
   DATE:        Dec 17 1996
   COMMENT:     TEXT figlet, for simple formatted text
*/
#ifndef TEXTH
#define TEXTH

#include "otools.h"

// Format
#define txtULINE 0x10
#define txtBOLD  0x20

// Look
#define txtBOX   0x01 // Keep text within the text-window

/*
  Text is stored as an array of copied strings within the figlet.
  One string contain four initial bytes:

  TEXT: oLEFT|oRIGHT|oHWIDE JJ...... Justification
        0-15                JJ..RRRR Rim
        txtBOLD             JJB.RRRR Bold
        txtULINE            JJBURRRR Underline

        0-255               FFFFFFFF Foreground index

        0-255               PPPPPPPP Paper color index

	0-255               CCCCCCCC Cursor color index

  The textbuffer starts with a default format (string 0) of:

    oLEFT|2                Plain text with 2 pixels rim
    ocFOREGROUND           The default textcolor
    ocBACKGROUND           The default paper color
    ocLIGHT                A good cursor color ?

  All strings have the following code of text formatting:

  !!    Exclamation
  !N    Normal text
  !B    Toggle bold text
  !U    Toggle underlined text
  !L    Left justification
  !R    Right justification
  !W    Block justification
  !M    Middle justification
  !fD3  Text color specification (hexadecimal)
  !b3F  Paper color specification (hexadecimal)
  !c1C  Cursor color specification (hexadecimal)
       
  */

class TEXT:public FIGLET {
protected:
  char    **text;                              // text
  ulong    *widths;                            // Width of each line
  ulong     text_count,                        // number of lines
            text_size,                         // size of textbuffer
            horisontal_scroll,
            vertical_scroll,
            max_width,
            max_height;
  PEN      *pen;                               // The pen (aka font)
  COLORSET *set;                               // The colorset
  uchar     look;                              // The look of the frame
  int       font_height;
  Pixmap    pixbuf;
  int       tab_spaces,pixmapcreated;

public:
        TEXT(WINDOW*,COLORSET*,PEN*,                // Window, Colorset and Pen
	     int,int,int,int,                                           // Size
	     char*,                               // Buffer with formatted text
	     uchar);                               // Look of frame around text
       ~TEXT(void);
  char  Token(uchar*,uchar*,uchar*,uchar*,ulong,ulong*,char*);     // Tokenizer
  ulong Width(uchar*,uchar*,uchar*,uchar*,ulong);    // Get endformat and width
  char  Append(uchar*,uchar*,uchar*,uchar*,char*); // Append a line to the text
  void  Format(char*);                                 // Formats the textinput
  void  Scroll(float,float);                          // Scroll the text-buffer
  void  Show(void);                                      // Show the textbuffer
  void  Refresh(void);                                      // Refresh the text
  void  Freshen(int ixa,int iya,int ixb,int iyb);            // Fresh a portion
  void  Draw(ulong new_x,ulong new_y,char redraw);             // Draw the text
  char  Load(uchar*,uchar*,uchar*,uchar*,char*);                 // Load a file
  void  Empty(void);                                         // Remove all text
  char  Resize(int,int,int,int);
  void  TextSize(ulong *,ulong *);
};

#endif /* TEXTH */
