/*
   NAME:     Xmix
   AUTHOR:   Ole Gunnar Westgaard
   DATE:     Jul 28 1995
   
*/

#include <stdio.h>
#include <stdlib.h>
#include "otools.h"
#include "xmix.xbm"
#include "figlets.h"

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>

DISPLAY  *dpy     = NULL;
COLORSET *cset1   = NULL,
         *cset2   = NULL,
         *cset3   = NULL;
WINDOW   *wnd     = NULL;

void Error(char *msg)
{
  printf("Xmix Error: %s\n",msg);
  otools.Stop();
  //if (dpy) dpy->Stop();
  exit(-1);
}

#ifdef LINUX
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#else
#warning Xmix does not support sound(cards) on this platform.
#endif

class VOLUME;
class RECORD;
class STEREO;

#ifndef LINUX
#define SOUND_MIXER_NRDEVICES 7
unsigned short VOLUMES[SOUND_MIXER_NRDEVICES] = {
  (85<<8 | 83),
  (05<<8 | 10),
  (55<<8 | 53),
  (73<<8 | 73),
  (95<<8 | 95),
  (90<<8 | 90),
  (25<<8 | 25)
};
char *SOUND_DEVICE_NAMES[SOUND_MIXER_NRDEVICES] = {
  "Master",
  "Mic",
  "Line",
  "CD",
  "Codec",
  "Midi",
  "Gain"
};
#endif

char     *dev[SOUND_MIXER_NRDEVICES] = SOUND_DEVICE_NAMES;
int       devmask = 0,
          recmask = 0,
          recsrc  = 0,
          mixer,
          n       = 0;
FIGLET   *rec[SOUND_MIXER_NRDEVICES];   // RECORD / TEXTLINE
VOLUME   *mix[SOUND_MIXER_NRDEVICES*2]; // Volumebar sliders
STEREO   *mno[SOUND_MIXER_NRDEVICES];   // Mono/Stereo buttons

//=============================================================================

#define CG "black:gray40:gray60:gray80:black:Blue4:red2"
#define CB "black:blue4:blue3:blue2:white:gray20:white"
#define CP "black:peachpuff4:peachpuff3:peachpuff1:royalblue4:black:gray20"

#define FILENAME ".Xmix"                                    // Name of savefile

//=============================================================================

class STEREO:public BUTTON { // Mono-stereo button
public:
  int flag;
  STEREO(int fl,WINDOW *w,COLORSET *cs,int a,int b,int c,int d,char *nm,
	 uchar tp,uchar th)
    : BUTTON(w,cs,a,b,c,d,nm,tp,th,0,NULL)
  {
    flag = fl;
  };
  char Trigger(ELEMENT*)
  {
    if (strcmp(Title(NULL),"locked") == 0)
      Title("free");
    else
      Title("locked");
    Refresh();
    return 0;
  };
};

//=============================================================================

class VOLUME:public SLIDER {
public:
  int flag;
  VOLUME(int fl,WINDOW *ow,COLORSET *oc,int a,int b,int c,int d,char *nm,
	 float vf,void(*cb)(SLIDER*,float,float))
    : SLIDER(ow,oc,NULL,a,b,c,d,nm,oHWIDE|oBOTTOM|sSNG,2,2,2,0,0.0,vf,cb)
  {
    flag = fl;
  };
};

void Update(SLIDER *s,float,float)
{
  int value,i = ((VOLUME*)s)->flag / 2; // sound device real number
  if (mno[i]->Pressed()) {
    if (((VOLUME*)s)->flag == (i*2))
      mix[i*2+1]->Values(0.0,mix[i*2]->Ver());
    else
      mix[i*2]->Values(0.0,mix[i*2+1]->Ver());
  }
  value = (int)(floor(mix[i*2]->Ver()*(float)100)) |
    ((int)(floor(mix[i*2+1]->Ver()*(float)100))<<8);
#ifdef LINUX
  if ( ioctl(mixer,MIXER_WRITE(i),&value)==-1 )
    Error("Update(); Could not do WRITE MIXER.");
  if ( ioctl(mixer,MIXER_READ(i),&value)==-1 )
    Error("Update(); Could not do MIXER_READ.");
#else
  // Do nothing, it's ok to change values
  VOLUMES[i] = value;
#endif
  mix[i*2  ]->Values(0.0,(float)( value     & 0x7f)/100.0);
  mix[i*2+1]->Values(0.0,(float)((value>>8) & 0x7f)/100.0);
  mix[i*2  ]->Draw();
  mix[i*2+1]->Draw();
};

//=============================================================================

class RECORD:public SWITCH {
public:
  int flag;
  RECORD(int fl,WINDOW *w,int a,int b,int c,int d,char *nm,char st)
    : SWITCH(w,a,b,c,d,nm,swLEFT|swDIAMOND|st,2,11,1,oLEFT|1,NULL)
  {
    flag = fl;
  };
  char Trigger(ELEMENT*)
  {
    // We have the flag allready
    if (! On())
      recsrc &= ~(1 << flag);
    else
      recsrc |= (1 << flag);

#ifdef LINUX
    if (ioctl(mixer,SOUND_MIXER_WRITE_RECSRC,&recsrc) == -1)
      Error("Record(); Could not do SOUND_MIXER_WRITE_RECSRC.");
    if (ioctl(mixer,SOUND_MIXER_READ_RECSRC,&recsrc) == -1)
      Error("Record(); Could not do SOUND_MIXER_READ_RECSRC.");
#endif // Nothing needs to be done, the new recsrc is ok

    for (int i=0; i<SOUND_MIXER_NRDEVICES; i++) {
      if ( ((1<<i) & recmask) )
	((RECORD*)rec[i])->State(((1<<i) & recsrc) ? swON : 0);
    }
    return 0;
  };
};

//=============================================================================

class TEXTLINE:public FIGLET {
public:
  char *name;
  TEXTLINE(WINDOW *w,int a,int b,int c,int d,char *nm)
    : FIGLET(w,a,b,c,d)
  {
    name = nm;
    window->Add(this);
  };
  ~TEXTLINE(void)
  {
    window->Del(this);
  };
  void Refresh(void)
  {
    int x,y;
    window->dpen()->Justify(&x,&y,xa,ya,xb,yb,0,name);
    window->dpen()->
      Foreground((*window->cset())[ocFOREGROUND]);
    window->dpen()->Write(x,y,name);
  };
};

//=============================================================================

void Warning(char *msg)
{
  printf("Xmix Warning: %s\n",msg);
}

//=============================================================================

class QUIT:public BUTTON {
public:
  QUIT(WINDOW *w,COLORSET *cs,int a,int b,int c,int d,char *nm)
    : BUTTON(w,cs,a,b,c,d,nm,butCLICK|butFRAME,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    otools.Stop();
    //window->dpy()->Stop();
    return 0;
  };
};

class SAVE:public BUTTON {
public:
  SAVE(WINDOW *w,COLORSET *cs,int a,int b,int c,int d,char *nm)
    : BUTTON(w,cs,a,b,c,d,nm,butCLICK|butFRAME,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    int   value,i;
    FILE *fout;
    //DBUG("Save%c%c%c",0,0,0);
    if ((fout = fopen(FILENAME,"wb"))) {
      for (i=0; i<SOUND_MIXER_NRDEVICES; i++) if ((1<<i) & devmask) {
	fprintf(fout,"%c",'A'+i);
	if (((1<<i) & recmask)) // make sure it's a rec-switch there
	  fprintf(fout,((RECORD*)rec[i])->On() ? "R" : "-");
	else
	  fprintf(fout," ");
	fprintf(fout,mno[i]->Pressed() ? "H" : " ");
#ifdef LINUX
	if (ioctl(mixer,MIXER_READ(i),&value) == -1)
	  Error("Save(); Could not do MIXER_READ.");
#else
	// Get back phony value
	value = VOLUMES[i];
#endif
	fprintf(fout,"%4X\n",value);
      }
      fclose(fout);
    } else Warning("Save(); Could not write savefile.");
    return 0;
  };
};

class LOAD:public BUTTON {
public:
  LOAD(WINDOW *w,COLORSET *cs,int a,int b,int c,int d,char *nm)
    : BUTTON(w,cs,a,b,c,d,nm,butCLICK|butFRAME,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    int   value,i;
    char  buf[8];
    FILE *fin;
    //DBUG("Load%c%c%c",0,0,0);
    if ((fin = fopen(FILENAME,"rb"))) {
      while (fread(buf,8,1,fin)) { // Read data for each device
	i = buf[0]-'A';
	if ((1<<i) & devmask) { // saved device in use here ?
	  if (buf[1] != ' ' && rec[i]) { // does saved device have rec switch ?
	    ((RECORD*)rec[i])->State(buf[1]=='R' ? swON : 0);
	    (RECORD*)rec[i]->Trigger((RECORD*)rec[i]);
	    rec[i]->Refresh();
	  }
	  mno[i]->State(buf[2]=='H' ? butIN : 0);
	  mno[i]->Title(buf[2]=='H' ? "locked" : "free");
	  mno[i]->Refresh();
	  sscanf(&buf[3],"%X",&value);
#ifdef LINUX
	  if ( ioctl(mixer,MIXER_WRITE(i),&value)==-1 )
	    Error("Load(); Could not do WRITE MIXER.");
	  if ( ioctl(mixer,MIXER_READ(i),&value)==-1 )
	    Error("Load(); Could not do MIXER_READ.");
#else
	  VOLUMES[i] = value;
#endif
	  mix[i*2  ]->Values(0.0,(float)( value     & 0x7f)/100.0);
	  mix[i*2+1]->Values(0.0,(float)((value>>8) & 0x7f)/100.0);
	  mix[i*2]->Refresh();
	  mix[i*2+1]->Refresh();
	}
      }
      fclose(fin);
    } else Warning("Load(); Could not read savefile.");
    return 0;
  };
};

//=============================================================================

void Init(void)
{
  int nr=0,value,i,size=((wnd->Width()-5)/n); // Size of each device in window
  //DBUG("Init%c%c%c",0,0,0);
  for (i=0; i<SOUND_MIXER_NRDEVICES; i++) if ((1<<i)&devmask) {
    // Get value of this volumebar
#ifdef LINUX
    if (ioctl(mixer,MIXER_READ(i),&value) == -1)
      Error("Init(); Could not do MIXER_READ.");
#else
    value = VOLUMES[i];
#endif

    FRAME *frame = new FRAME(wnd,nr*size+3,24,(nr+1)*size,-2,fSNG|fIN,2,0);
    
    mix[i*2] = new VOLUME(i*2,wnd,cset2,
			  nr*size+5,26,
			  nr*size+size/2+1,wnd->Height()-35,
			  NULL,((float)(value & 0x7f)/(float)100),Update);
    mix[i*2+1]=new VOLUME(i*2+1,wnd,cset2,
			  nr*size+size/2+2,26,
			  (nr+1)*size-1,wnd->Height()-35,
			  NULL,((float)((value>>8)&0x7f)/(float)100),Update);
    frame->Add(mix[i*2]);
    frame->Add(mix[i*2+1]);

    // Initialize Recording Switches
    if ((1<<i)&recmask) {
      rec[i] = new RECORD(i,wnd,
			  nr*size+5,wnd->Height()-35,
			  (nr+1)*size-2,wnd->Height()-19,
			  dev[i],((1<<i) & recsrc) ? swON : 0);
    } else {
      rec[i] = new TEXTLINE(wnd,
			    nr*size+5,wnd->Height()-35,
			    (nr+1)*size-2,wnd->Height()-19,
			    dev[i]);
    }

    frame->Add(rec[i]);

    // Initialize free/locked buttons (stereo/mono sliders)

    mno[i] = new STEREO(i,wnd,cset3,nr*size+5,wnd->Height()-18,(nr+1)*size-2,
			-4,"free",butHOLD,2);

    frame->Add(mno[i]);

    nr++;
  }
}  

void EnterWindow(WINDOW*,int,int)
{
  int value;
  float v;
  //IBUG("EnterWindow()%c%c%c",0,0,0);
  for (int i=0; i<SOUND_MIXER_NRDEVICES; i++) if ((1<<i)&devmask) {
    // Get value of this volumebar
#ifdef LINUX
    if (ioctl(mixer,MIXER_READ(i),&value)==-1)
      Error("EnterWindow(); Could not do MIXER_READ.");
#else
    value = VOLUMES[i];
#endif
    // Set volumebars 

    v=(float)(value & 0x7f)/(float)100;

    if (v != mix[i*2]->Ver()) {
      mix[i*2]->Values(0.0,v);
      mix[i*2]->Draw();
    }

    v = (float)((value>>8)&0x7f)/(float)100;

    if (v != mix[i*2+1]->Ver()) {
      mix[i*2+1]->Values(0.0,v);
      mix[i*2+1]->Draw();
    }

    // Set recbuttons

#ifdef LINUX
    if ( ioctl(mixer,SOUND_MIXER_READ_RECSRC,&recsrc)==-1 )
      Error("EnterWindow(); Could not do SOUND_MIXER_READ_RECSRC.");
#endif

    if ((1<<i)&recmask) {
      ((SWITCH*)rec[i])->State(((1<<i) & recsrc) ? swON : 0);
      ((SWITCH*)rec[i])->Draw();
    }

  }
  //OBUG("EnterWindow()%c%c%c",0,0,0);
}

class ENTERWINDOW : public WINDOW {
public:
  ENTERWINDOW(COLORSET *cs,int x,int y,int w,int h,char *fn)
    :WINDOW(0,NULL,cs,"Xmix by olew@ifi.uio.no","Xmix",x,y,w,h,w,h,w,h,
	    0,xmix_bits,xmix_width,xmix_height,fn) {};
  void Enter(int x,int y) {
    EnterWindow(this,x,y);
  };
};

int main(int,char**)
{
#ifdef LINUX
  char devmixer[30] = "/dev/mixer"; // Name of sound-device
#endif

  // Initialize the sounddevice
#ifdef LINUX
  if ( (mixer=open(devmixer,O_RDWR))<0 )
    Error("main(); Could not open mixerdevice.");
  if ( ioctl(mixer,SOUND_MIXER_READ_DEVMASK,&devmask)==-1 )
    Error("main(); Could not do SOUND_MIXER_READ_DEVMASK.");
  if ( ioctl(mixer,SOUND_MIXER_READ_RECMASK,&recmask)==-1 )
    Error("main(); Could not do SOUND_MIXER_READ_RECMASK.");
  if ( ioctl(mixer,SOUND_MIXER_READ_RECSRC,&recsrc)==-1 )
    Error("main(); Could not do SOUND_MIXER_READ_RECSRC.");
#else
  devmask = 0x7F;
  recmask = 0x3D;
  recsrc  = 0x31;
#endif

  // Count number of accessible devices
  for (int i=0; i<SOUND_MIXER_NRDEVICES; i++)
    if ((1<<i)&devmask) n++;

  // DISPLAY
  dpy = new DISPLAY(NULL);
  if (!dpy->xdpy()) Error("main(); Cannot initialize display!");

  // COLORS
  cset1 = new COLORSET(dpy->cmap(),CG);
  cset2 = new COLORSET(dpy->cmap(),CB);
  cset3 = new COLORSET(dpy->cmap(),CP);

  // WINDOW
  wnd = new ENTERWINDOW(cset1,5,25,n*52+5,140,"6x10");
  
  // BUTTONS
  FRAME *f = new FRAME(wnd,2,2,wnd->Width()-145,21,fSNG|fIN,2,0);
#ifdef LINUX
  f->Add(new TEXTLINE(wnd,4,4,wnd->Width()-147,19,"Xmix 2.0"));
#else
  f->Add(new TEXTLINE(wnd,4,4,wnd->Width()-147,19,"Xmix 2.0 - demonstration"));
#endif

  LOAD *load= 
  new LOAD(wnd,cset3,-141,2,-97,21,"Load");
  new SAVE(wnd,cset3,- 95,2,-53,21,"Save");
  new QUIT(wnd,cset3,- 51,2,- 5,21,"Quit");

  Init();
  load->Trigger(load); // Load .Xmix

  otools.EventLoop();
  //dpy->EventLoop();
  delete dpy;
  close(mixer);
}
