/*
   NAME:	xcol.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jul 31 1996
   COMMENT:	Xcolor

   xcol [-dpy display] [-num number_of_colors]

*/

#define METHOD2

#include <stdio.h>
#include "otools.h"
#include "figlets.h"
#include "xcol.xbm"

char  MYCOLS[512] = "black:salmon4:salmon3:salmon1:black:blue4:pink";
char  MYFONT[256] = "-*-*-*-*-*-*16-*-*-*-*-*-iso8859-1";
char  RGBFILE[512] = "/usr/X11R6/lib/X11/rgb.txt";
int   NUMFRAMES    =   4,
      NUMCOLORS    =  80,
      WIDTH        = 630,
      HEIGHT       = 440;
char *DPY          = NULL;
char  NONAME       = 0;
char  MIDDLE       = 1;
char  OWNCMAP      = 0;

DISPLAY  *dpy;
WINDOW   *wnd;
COLORMAP *own = NULL;
COLORSET *set = NULL,*wset= NULL;
BUTTON   *SHOW;
#ifdef METHOD2
FRAME    *FR;
#else
FRAME    *FR[10];
#endif
LIST      colors(30,20),
          cfigs(30,20);

char eq(char *a,char *b,char *c,char *d,char *e)
{
  if (!a) return 0;
  if (b && (strcmp(a,b)==0)) return 1;
  if (c && (strcmp(a,c)==0)) return 1;
  if (d && (strcmp(a,d)==0)) return 1;
  if (e && (strcmp(a,e)==0)) return 1;
  return 0;
}

void ParseOptions(int ac,char *v[])
{
  int i = 1;
  while (i<ac) {
    if (v[i][0]=='+') {
      if (eq(v[i],"+noname","+NONAME","+nn","+NN")) {
	NONAME=0;
      }
      else if (eq(v[i],"+middle","+MIDDLE","+md","+MD")) {
	MIDDLE=0;
      }
    }
    if (v[i][0]=='-') {
      if (eq(v[i],"-noname","-NONAME","-nn","-NN")) {
	NONAME=1;
      }
      else if (eq(v[i],"-middle","-MIDDLE","-md","-MD")) {
	MIDDLE=1;
      }
      else if (eq(v[i],"-owncmap","-OWNCMAP","-ocn","-OCM")) {
	OWNCMAP=1;
      }
      else if (eq(v[i],"-font","-FONT","-fn","-FN")) {
	strcpy(MYFONT,v[++i]);
      }
      else if (eq(v[i],"-colors","-COLORS","-co","-CO")) {
	strcpy(MYCOLS,v[++i]);
      }
      else if (eq(v[i],"-display","-DISPLAY","-dpy","-DPY")) {
	DPY = v[++i];
      }
      else if (eq(v[i],"-numcolors","-NUMCOLORS","-nc","-NC")) {
	if (!sscanf(v[++i],"%d",&NUMCOLORS)) NUMCOLORS=30; else
	  if (NUMFRAMES>10) NUMFRAMES=10;
      }
      else if (eq(v[i],"-frames","-FRAMES","-fr","-FR")) {
	if (!sscanf(v[++i],"%d",&NUMFRAMES)) NUMFRAMES=2;
      }
      else if (eq(v[i],"-geometry","-GEOMETRY","-geo","-GEO")) {
	sscanf(v[++i],"%dx%d",&WIDTH,&HEIGHT);
      }
      else if (eq(v[i],"-rgb","-RGB","-rgbfile","-RGBFILE")) {
	strcpy(RGBFILE,v[++i]);	
      }
      else if (eq(v[i],"-h","--help","-help","-H")) {
	printf("\n");
	printf("  xcol [-fn fnt] [-co \"color:list\"] [-nc num] [-geo WxH]\n");
	printf("       [-dpy display] [-h] [-/+noname|-/+middle]\n");
        printf("       [-fr num] [-rgb <file>] [-owncmap]\n");
	printf("\n");
	printf("  Written by Ole Gunnar Westgaard - olew@ifi.uio.no\n");
	printf("\n");
	printf("  Pressing left button on colors selects text color\n");
	printf("  Click right button on three colors to change background\n");
	printf("  Pressing middle button adds colorname to colortable\n");
	printf("\n");
	exit(0);
      }
    }
    ++i;
  }
  NUMCOLORS = (NUMCOLORS/NUMFRAMES)*NUMFRAMES;
  if (NUMCOLORS<NUMFRAMES) NUMCOLORS=NUMFRAMES;
}

//=============================================================================

class COLOR:public ELEMENT {
private:
  char color[8];
  char *name;
public:
  COLOR(int,int,int,int,char*);
  virtual ~COLOR(void);
  void hex(int,char*);
  char *hexcol(void) { return color; };
  char *charcol(void) { return name; };
  void Change(char*);
};

COLOR::COLOR(int num,int r,int g,int b,char *n)
{
  name = new char[strlen(n)+1];
  strcpy(name,n);
  for (int i=0; name[i]!='\0'; i++) {
    if (name[i]=='\n') name[i]='\0';
    if (name[i]=='\r') name[i]='\0';
  }
  //printf("%ld %s\n",key,name);
  sprintf(color,"#%2X%2X%2X",r,g,b);
  for (int i=0; color[i]!='\0'; i++) if (color[i]==' ') color[i]='0';
  colors.add(this,(ulong)num);                         // sorted by it's number
}

void COLOR::Change(char *n)
{
  // printf("Changing %s to %s",name,n);
  delete [] name;
  name = new char[strlen(n)+1];
  strcpy(name,n);
  for (int i=0; name[i]!='\0'; i++) {
    if (name[i]=='\n') name[i]='\0';
    if (name[i]=='\r') name[i]='\0';
  }
}

COLOR::~COLOR(void)
{
  delete [] name;
  colors.remove(this);
}

//=============================================================================

class CFIG:public FIGLET {
private:
  int color;
public:
  CFIG(WINDOW*,int);
  virtual ~CFIG(void);
  virtual void Refresh(void);
  virtual char Press(uchar,int,int);
  virtual void Draw(void);
};

CFIG::CFIG(WINDOW *w,int c)
  :FIGLET(w,0,0,0,0)
{
  color = c;
  window->Activate(this);
  cfigs.append(this);
}

CFIG::~CFIG(void)
{
  window->DeActivate(this);
  cfigs.remove(this);
}

void CFIG::Refresh(void)
{
  if (NONAME || MIDDLE) {
    window->dpen()->Foreground((*set)[color]);
    window->dpen()->Fbox(xa,ya,xb,yb);
  } else {
    window->dpen()->Foreground((*set)[color]);
    window->dpen()->Fbox(xa,ya,xa+((yb-ya+1)/3)*3,yb);
  }
  Draw();
}

void CFIG::Draw(void)
{
  char *t = set->name(color);
  int x,y;
  if (NONAME) return;
  window->dpen()->Foreground((*set)[0]);
  if (MIDDLE)
    window->dpen()->Justify(&x,&y,xa,ya,xb,yb,oMIDDLE,t);
  else
    window->dpen()->Justify(&x,&y,xa+((yb-ya+1)/3)*3+1,
				  ya,xb,yb,oLEFT|2,t);
  window->dpen()->Write(x,y,t);
}

char CFIG::Press(uchar button,int,int)
{
  if (button==1) {
    CFIG *f;
    set->Change(0,set->name(color));
    for (ulong i=1; (f = (CFIG*)cfigs[i]); ++i) f->Draw();
  } else if (button==3) {
    static int i=1;
    window->cset()->Change(i,set->name(color));
    if ((++i)>3) {
      window->Refresh(0,0,window->Width()-1,window->Height()-1);
      i=1;
    }
  } else {
    char t[4096];
    strcpy(t,SHOW->Title(NULL));
    if (strlen(t)>4000) return 0;
    strcat(t,":");
    strcat(t,set->name(color));
    SHOW->Title(t);
    SHOW->Refresh();
  }
  return 0;
}

//=============================================================================

void Quit(char *msg,char *m)
{
  if (msg) {
    fprintf(stderr,"Abort: "); fprintf(stderr,msg,m); fprintf(stderr,"\n");
  }
  while (colors.number_of_elements() > 0)
    delete (COLOR*)colors[1];            // Will remove itself from colors-list
  exit(0);
}

int  isof(char c,char *s)
{
  for (int i=0; s[i]!='\0'; i++) if (s[i] == c) return 1;
  return 0;
}

int color_index = 0,
    color_max   = 0;

void ParseColors(char *filename)
{
  COLOR *last = NULL;
  int r,g,b,or=-1,og=-1,ob=-1;
  int index=0;
  char line[128];
  FILE *fin = fopen(filename,"r");
  if (!fin) Quit("Cannot open RGB database %s",filename);
  while (fgets(line,127,fin) != '\0') {
    if ((line[0] != '!')) {
      if (sscanf(line,"%d %d %d",&r,&g,&b)==3) {
	int i=0;
	while (isof(line[i],"0123456789 \t")) i++;
	if ((or!=r) || (og!=g) || (ob!=b)) {
	  last = new COLOR(index,r,g,b,line+i);
	  or = r; og = g; ob = b;
	  index++;
	} else last->Change(line+i);
      }
    }
  }
  color_max = index-1;
  if (NUMCOLORS>index) NUMCOLORS = (index/NUMFRAMES)*NUMFRAMES;
  fclose(fin);
}

void CreateColors(void)
{
  if (!set) {
    char c[4096]= "black";
    for (int i=0; i<NUMCOLORS; i++) strcat(c,":red");
    set = new COLORSET(dpy->cmap(),c);
  }
  for (int i=0; i<NUMCOLORS; i++) {
    COLOR *c = (COLOR*) colors.find((ulong)(color_index+i));
    if (c) set->Change(i+1,c->charcol());
  }
}

//=============================================================================

class QUITBUTTON : public BUTTON {
public:
  QUITBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    otools.Stop();
    return 0;
  };
};

class TELLBUTTON : public BUTTON {
public:
  TELLBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    int i = 0;
    char line[4096];
    strcpy(line,SHOW->Title(NULL));
    while (line[i]==':') ++i;
    printf("\"%s\"\n",line+i);
    return 0;
  }
};

class CLEARBUTTON : public BUTTON {
public:
  CLEARBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    Title("");
    Refresh();
    return 0;
  };
};

class NEXTBUTTON : public BUTTON {
public:
  NEXTBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    if ((color_index+NUMCOLORS-1)>=color_max) return 0;
    color_index+=NUMCOLORS;
    if ((color_index+NUMCOLORS-1)>color_max) color_index = color_max-NUMCOLORS+1;
    CreateColors();
#ifdef METHOD2
    FR->Refresh();
#else
    for (int j=0; j<NUMFRAMES; j++) FR[j]->Refresh();
#endif
    return 0;
  };
};

class PLUSBUTTON : public BUTTON {
public:
  PLUSBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    if ((color_index+NUMCOLORS-1)>=color_max) return 0;
    color_index+=NUMCOLORS/NUMFRAMES;
    if ((color_index+NUMCOLORS-1)>color_max) color_index = color_max-NUMCOLORS+1;
    CreateColors();
#ifdef METHOD2
    FR->Refresh();
#else
    for (int j=0; j<NUMFRAMES; j++) FR[j]->Refresh();
#endif
    return 0;
  };
};

class PREVBUTTON : public BUTTON {
public:
  PREVBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    if (color_index<=0) return 0;
    color_index-=NUMCOLORS;
    if (color_index<0) color_index = 0;
    CreateColors();
#ifdef METHOD2
    FR->Refresh();
#else
    for (int j=0; j<NUMFRAMES; j++) FR[j]->Refresh();
#endif
    return 0;
  };
};

class MINUSBUTTON : public BUTTON {
public:
  MINUSBUTTON(WINDOW *w,int a,int b,int c,int d,char *s)
    :BUTTON(w,a,b,c,d,s,butFRAME|2,2,0,NULL) {};
  char Trigger(ELEMENT*)
  {
    if (color_index<=0) return 0;
    color_index-=NUMCOLORS/NUMFRAMES;
    if (color_index<0) color_index = 0;
    CreateColors();
#ifdef METHOD2
    FR->Refresh();
#else
    for (int j=0; j<NUMFRAMES; j++) FR[j]->Refresh();
#endif
    return 0;
  };
};

int main(int argc,char *argv[])
{ 
  ParseOptions(argc,argv);
  ParseColors(RGBFILE);

  dpy = new DISPLAY(DPY);
  if (!dpy->xdpy()) Quit("Cannot initialize display %s",DPY);

  wset = new COLORSET(dpy->cmap(),MYCOLS);

  wnd = new WINDOW(wset,"Colors",MYFONT,50,50,WIDTH,HEIGHT,
		   xcol_bits,xcol_width,xcol_height);
  if (!wnd->dpy()) Quit("Cannot open window on display %s",DPY);

  int m = wnd->Width()/6;

  new  QUITBUTTON(wnd,  5  ,5,m  -2,30,"Quit");
  new MINUSBUTTON(wnd,m  +2,5,m*2-2,30,"-");
  new  PLUSBUTTON(wnd,m*4+2,5,m*5-2,30,"+");
  new  PREVBUTTON(wnd,m*2+2,5,m*3-2,30,"Previous");
  new  NEXTBUTTON(wnd,m*3+2,5,m*4-2,30,"Next");
  new  TELLBUTTON(wnd,m*5+2,5,wnd->Width()-6,30,"Tell");

  SHOW = new CLEARBUTTON(wnd,
			 5,wnd->Height()-31,wnd->Width()-6,wnd->Height()-6,
			 "");

  CreateColors();

#ifdef METHOD2

  FR = new FRAME(wnd,5,35,wnd->Width()-6,wnd->Height()-36,bSNG|fSNG|fIN,3,0);
  FR->Stacking(NUMFRAMES,NUMCOLORS/NUMFRAMES,1,1,frVERFIRST,3);
  for (int i=0; i<NUMCOLORS; i++)
    FR->Add(new CFIG(wnd,i+1));

#else

  float w = (float)wnd->Width()/(float)NUMFRAMES;

  for (int i=0; i<NUMFRAMES; i++) {
    int ax = (int)(w * (float)i)+3,
        bx = (int)(w * (float)(i+1))-3;
    FR[i] = new FRAME(wnd,ax,35,bx,wnd->Height()-36,bSNG|fSNG|fIN,2,0);
    FR[i]->Stacking(1,NUMCOLORS/NUMFRAMES,1,1,0,0);

    for (int j=0; j<(NUMCOLORS/NUMFRAMES); j++)
      FR[i]->Add(new CFIG(wnd,(i*(NUMCOLORS/NUMFRAMES))+j+1));
  }

  dpy->flush();

#endif // METHOD2

  otools.EventLoop();
  // dpy->EventLoop();

  delete dpy;

  Quit(NULL,NULL);
}

