/*
   NAME:	fun.cc
   AUTHOR:	Ole Gunnar Westgaard TFU <olew@dina>
   DATE:	Jun 28 1996
   COMMENT:	Otools 2.0 - testbed program
*/

#include <stdio.h>
#include "otools.h"
#include "figlets.h"
#include "icon.xbm"

class RESIZABLEWINDOW;
//=============================================================================

int c,n=0;
char **a;

char *font = "7x13bold";

DISPLAY         *display = NULL;
RESIZABLEWINDOW *window  = NULL;
COLORSET        *bg      = NULL;
TEXT            *text    = NULL;

//=============================================================================

void horisontal_scroll(SLIDER*,float h,float)
{
  text->Scroll(h,-1.0);
}

void vertical_scroll(SLIDER*,float,float v)
{
  text->Scroll(-1.0,1.0-v);
}

void Quit_Message(char*);
void Next_File(char*);

void parse(void)
{
  int m=1;
  while ((m<c) && (a[m][0]=='-')) {
    switch (a[m][1]) {
    case 'f':
    case 'F':
      ++m;
      font = a[m];
      break;
    }
    ++m;
  }
  n = m-1;
}

//=============================================================================

class RESIZABLEWINDOW : public WINDOW {
public:
  MENU     *menu;
  FRAME    *frame;
  SLIDER   *hors,*vers;
  COLORSET *tx;

  RESIZABLEWINDOW(COLORSET *cs,char *ne,int mw,int mh,int w,int h,int xw,
		  int xh,char *ft)
    : WINDOW()
  {
    IBUG(D_WINDOW|D_MED,"new RESIZABLEWINDOW[%p](\"%s\")",this,ne);
    if (Create(display,cs->cmap(),cs,0,
	       ne,ne,10,10,mw,mh,w,h,xw,xh,
	       0,icon_bits,icon_width,icon_height,ft,0,NULL,0)) {
      DBUG(D_WINDOW|D_LOW,"Mapping and registering window to display '%s'",
	   display->name());
      
      tx = new COLORSET(display->cmap(),"\
black:gray20:gray50:gray80:black:MidnightBlue:MediumBlue:red:yellow:cyan:\
green:OrangeRed4:wheat1:DeepPink4:purple4");
      menu = new MENU(this,tx,0,SUB("File",OPT2("Next",Next_File,
						"Quit",Quit_Message)));
      
      text = new TEXT(this,tx,pen,10,40,-41,-41,NULL,txtBOX);
      frame = new FRAME(this,tx,5,menu->pixel_height()+4,-16,-16,
			bSNG|fSNG|fIN,2,0,1,2,3,4);
      frame->Stacking(1,1,1,1,0,0);
      frame->Add(text);
      vers = new SLIDER(this,tx,pen,-13,menu->pixel_height()+4,-3,-16,
			NULL,oHWIDE|sSNG,
			2,15,15,0,0.0,1.0,vertical_scroll);
      hors = new SLIDER(this,tx,pen,5,-13,-16,-3,NULL,oVWIDE|sSNG,
			2,15,15,0,0.0,1.0,horisontal_scroll);
      
      display->Add(this);                               // Enroll me with display
      XMapWindow(display->xdpy(),window);
      OBUG(D_MED,"Created.");
    } else {
      WARN("new RESIZABLEWINDOW[%p](maximal creator) Cannot create!",this);
      OBUG(D_MED,"Not Created!");
    }
  };
  
  void Size(void)
  {
    frame->Del(text);
    frame->Resize(5,menu->pixel_height()+4,-16,-16);
    frame->Stacking(1,1,1,1,0,0);
    frame->Add(text);
    vers->Resize(-13,menu->pixel_height()+4,-3,-16);
    hors->Resize(5,-13,-16,-3);
    menu->Resize();
  };

  void NextText(void)
  {
    ++n;
    uchar fm=oLEFT,bg=2,fg=4,cr=6;
    text->Empty();
    if (n<c) 
      text->Load(&fm,&fg,&bg,&cr,a[n]);
    text->Scroll(0.0,0.0);
    vers->Values(0.0,1.0);
    hors->Values(0.0,0.0);
    vers->Draw();
    hors->Draw();
  };

};


//=============================================================================

void Quit_Message(char *msg)
{
  fprintf(stderr,"Quitting: %s\n",msg);
  otools.Stop();
}

void Abort(char *msg)
{
  Quit_Message(msg);
  exit(0);
}

void Next_File(char*)
{
  window->NextText();
}

//=============================================================================

int main(int argc,char *argv[])
{
  c = argc;
  a = argv;
  parse();

  if (argc<2) Abort("Need arguments: [-f font] <files...>");

  // DISPLAY ------------------------------------------------------------------
  display = new DISPLAY(NULL);
  if (!display->xdpy()) Abort("Cannot initialize X11 display");

  // COLORSETS ----------------------------------------------------------------
  bg = new COLORSET(display->cmap(),
		    "black:RoyalBlue4:RoyalBlue3:RoyalBlue1:black:black:white"
		    );
  
  // WINDOW -------------------------------------------------------------------
  window = new RESIZABLEWINDOW(bg,"Text viewer",400,300,500,600,
			       display->Width(),display->Height(),font);
  if (!window->dpy()) Abort("Cannot create a window");

  window->NextText();
  display->Flush();

  otools.EventLoop();

  delete display;
  
  return 0;
}
